@@ -1,5 +1,13 @@
 #include "directorytree.h"
 
+/*
+
+directorytree.c
+
+ Related to library / directory structure.
+
+*/
+
 static int lastUsedId = 0;
 
 typedef void (*TimeoutCallback)(void);
@@ -46,7 +54,7 @@ void setFullPath(FileSystemEntry *entry, const char *parentPath, const char *ent
                 return;
         }
 
-        size_t fullPathLength = strlen(parentPath) + strlen(entryName) + 2; // +2 for '/' and '\0'
+        size_t fullPathLength = strnlen(parentPath, MAXPATHLEN) + strnlen(entryName, MAXPATHLEN) + 2; // +2 for '/' and '\0'
 
         entry->fullPath = (char *)malloc(fullPathLength);
         if (entry->fullPath == NULL)
@@ -56,30 +64,6 @@ void setFullPath(FileSystemEntry *entry, const char *parentPath, const char *ent
         snprintf(entry->fullPath, fullPathLength, "%s/%s", parentPath, entryName);
 }
 
-void displayTreeSimple(FileSystemEntry *root, int depth)
-{
-        for (int i = 0; i < depth; ++i)
-        {
-                printf("  ");
-        }
-
-        printf("%s", root->name);
-        if (root->isDirectory)
-        {
-                printf(" (Directory)\n");
-                FileSystemEntry *child = root->children;
-                while (child != NULL)
-                {
-                        displayTreeSimple(child, depth + 1);
-                        child = child->next;
-                }
-        }
-        else
-        {
-                printf(" (File)\n");
-        }
-}
-
 void freeTree(FileSystemEntry *root)
 {
         if (root == NULL)
@@ -146,59 +130,6 @@ int removeEmptyDirectories(FileSystemEntry *node)
         return numEntries;
 }
 
-char *stringToUpperWithoutSpaces(const char *str)
-{
-        if (str == NULL)
-        {
-                return NULL;
-        }
-
-        size_t len = strlen(str);
-        char *result = (char *)malloc(len + 1);
-        if (result == NULL)
-        {
-                return NULL;
-        }
-
-        size_t resultIndex = 0;
-        for (size_t i = 0; i < len; ++i)
-        {
-                if (!isspace((unsigned char)str[i]))
-                {
-                        result[resultIndex++] = toupper((unsigned char)str[i]);
-                }
-        }
-
-        result[resultIndex] = '\0';
-
-        return result;
-}
-
-int compareLibEntries(const struct dirent **a, const struct dirent **b)
-{
-        char *nameA = stringToUpperWithoutSpaces((*a)->d_name);
-        char *nameB = stringToUpperWithoutSpaces((*b)->d_name);
-
-        if (nameA[0] == '_' && nameB[0] != '_')
-        {
-                free(nameA);
-                free(nameB);
-                return 1;
-        }
-        else if (nameA[0] != '_' && nameB[0] == '_')
-        {
-                free(nameA);
-                free(nameB);
-                return -1;
-        }
-
-        int result = strcmp(nameB, nameA);
-        free(nameA);
-        free(nameB);
-
-        return result;
-}
-
 int readDirectory(const char *path, FileSystemEntry *parent)
 {
 
@@ -210,7 +141,8 @@ int readDirectory(const char *path, FileSystemEntry *parent)
         }
 
         struct dirent **entries;
-        int dirEntries = scandir(path, &entries, NULL, compareLibEntries);
+        int dirEntries = scandir(path, &entries, NULL, compareLibEntriesReversed);
+
         if (dirEntries < 0)
         {
                 perror("Error scanning directory entries");
@@ -245,7 +177,7 @@ int readDirectory(const char *path, FileSystemEntry *parent)
                                 isDirectory = false;
                         }
 
-                        char exto[6];
+                        char exto[100];
                         extractExtension(entry->d_name, sizeof(exto) - 1, exto);
 
                         int isAudio = match_regex(&regex, exto);
@@ -389,7 +321,7 @@ FileSystemEntry *reconstructTreeFromFile(const char *filename, const char *start
                                         }
                                         free(nodes);
                                         fclose(file);
-                                        exit(EXIT_FAILURE);
+                                        exit(1);
                                 }
                                 nodes = tempNodes;
                         }
@@ -400,7 +332,7 @@ FileSystemEntry *reconstructTreeFromFile(const char *filename, const char *start
                                 perror("Failed to allocate node");
 
                                 fclose(file);
-                                exit(EXIT_FAILURE);
+                                exit(1);
                         }
                         node->id = id;
                         node->name = strdup(name);
@@ -445,78 +377,72 @@ FileSystemEntry *reconstructTreeFromFile(const char *filename, const char *start
         return root;
 }
 
-int min(int a, int b, int c)
-{
-        if (a <= b && a <= c)
-                return a;
-        if (b <= a && b <= c)
-                return b;
-        return c;
-}
-
 #ifdef __GNUC__
+#ifndef __APPLE__
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
 #endif
+#endif
 // Calculates the Levenshtein distance.
 // The Levenshtein distance between two strings is the minimum number of single-character edits
 // (insertions, deletions, or substitutions) required to change one string into the other.
-int levenshteinDistance(const char *s1, const char *s2)
+int utf8_levenshteinDistance(const char *s1, const char *s2)
 {
-        int len1 = strlen(s1);
-        int len2 = strlen(s2);
-        int **d = (int **)malloc((len1 + 1) * sizeof(int *));
-        for (int i = 0; i <= len1; i++)
-        {
-                d[i] = (int *)malloc((len2 + 1) * sizeof(int));
-                for (int j = 0; j <= len2; j++)
-                {
-                        d[i][j] = 0;
-                }
-        }
+        // Get the length of s1 and s2 in terms of characters, not bytes
+        int len1 = g_utf8_strlen(s1, -1);
+        int len2 = g_utf8_strlen(s2, -1);
 
-        for (int i = 0; i <= len1; i++)
-        {
-                d[i][0] = i;
-        }
+        // Allocate a 2D matrix (only two rows at a time are needed)
+        int *prevRow = (int *)malloc((len2 + 1) * sizeof(int));
+        int *currRow = (int *)malloc((len2 + 1) * sizeof(int));
+
+        // Initialize the first row (for empty s1)
         for (int j = 0; j <= len2; j++)
         {
-                d[0][j] = j;
+                prevRow[j] = j;
         }
 
-        for (int i = 1; i <= len1; i++)
+        // Iterate over the characters of both strings
+        const char *p1 = s1;
+        for (int i = 1; i <= len1; i++, p1 = g_utf8_next_char(p1))
         {
-                for (int j = 1; j <= len2; j++)
+                currRow[0] = i;
+                const char *p2 = s2;
+                for (int j = 1; j <= len2; j++, p2 = g_utf8_next_char(p2))
                 {
-                        int cost = (s1[i - 1] == s2[j - 1]) ? 0 : 1;
-                        d[i][j] = min(d[i - 1][j] + 1,         // deletion
-                                      d[i][j - 1] + 1,         // insertion
-                                      d[i - 1][j - 1] + cost); // substitution
+                        // Compare Unicode characters using g_utf8_get_char
+                        gunichar c1 = g_utf8_get_char(p1);
+                        gunichar c2 = g_utf8_get_char(p2);
+
+                        int cost = (c1 == c2) ? 0 : 1;
+
+                        // Fill the current row with the minimum of deletion, insertion, or substitution
+                        currRow[j] = MIN(prevRow[j] + 1,              // deletion
+                                         MIN(currRow[j - 1] + 1,      // insertion
+                                             prevRow[j - 1] + cost)); // substitution
                 }
+
+                // Swap rows (current becomes previous for the next iteration)
+                int *temp = prevRow;
+                prevRow = currRow;
+                currRow = temp;
         }
 
-        int distance = d[len1][len2];
-        for (int i = 0; i <= len1; i++)
-        {
-                free(d[i]);
-        }
-        free(d);
+        // The last value in prevRow contains the Levenshtein distance
+        int distance = prevRow[len2];
+
+        // Free the allocated memory
+        free(prevRow);
+        free(currRow);
+
         return distance;
 }
+
 #ifdef __GNUC__
+#ifndef __APPLE__
 #pragma GCC diagnostic pop
 #endif
-
-// Returns a new string that is lowercase of str
-char *strLower(char *str)
-{
-        char *lowerStr = strdup(str);
-        for (char *p = lowerStr; *p; ++p)
-        {
-                *p = tolower(*p);
-        }
-        return lowerStr;
-}
+#endif
 
 // Traverses the tree and applies fuzzy search on each node
 void fuzzySearchRecursive(FileSystemEntry *node, const char *searchTerm, int threshold, void (*callback)(FileSystemEntry *, int))
@@ -527,10 +453,10 @@ void fuzzySearchRecursive(FileSystemEntry *node, const char *searchTerm, int thr
         }
 
         // Convert search term, name, and fullPath to lowercase
-        char *lowerSearchTerm = strLower((char *)searchTerm);
-        char *lowerName = strLower(node->name);
+        char *lowerSearchTerm = g_utf8_casefold((char *)searchTerm, -1);
+        char *lowerName = g_utf8_casefold(node->name, -1);
 
-        int nameDistance = levenshteinDistance(lowerName, lowerSearchTerm);
+        int nameDistance = utf8_levenshteinDistance(lowerName, lowerSearchTerm);
 
         // Partial matching with lowercase strings
         if (strstr(lowerName, lowerSearchTerm) != NULL)
