@@ -1,3 +1,6 @@
+#ifndef EVENTS_H
+#define EVENTS_H
+
 #define MAX_SEQ_LEN 1024    // Maximum length of sequence buffer
 
 enum EventType
@@ -9,7 +12,6 @@ enum EventType
         EVENT_NEXT,
         EVENT_PREV,
         EVENT_QUIT,
-        EVENT_TOGGLECOVERS,
         EVENT_TOGGLEREPEAT,
         EVENT_TOGGLEVISUALIZER,
         EVENT_TOGGLEBLOCKS,
@@ -22,10 +24,11 @@ enum EventType
         EVENT_SHOWKEYBINDINGS,
         EVENT_SHOWPLAYLIST,
         EVENT_SHOWSEARCH,
+        EVENT_SHOWRADIOSEARCH,
         EVENT_GOTOSONG,
         EVENT_GOTOBEGINNINGOFPLAYLIST,
         EVENT_GOTOENDOFPLAYLIST,
-        EVENT_TOGGLE_PROFILE_COLORS,
+        EVENT_TOGGLEPROFILECOLORS,
         EVENT_SCROLLNEXT,
         EVENT_SCROLLPREV,
         EVENT_SEEKBACK,
@@ -36,7 +39,13 @@ enum EventType
         EVENT_PREVPAGE,
         EVENT_REMOVE,
         EVENT_SEARCH,
-        EVENT_TABNEXT
+        EVENT_NEXTVIEW,
+        EVENT_PREVVIEW,
+        EVENT_CLEARPLAYLIST,
+        EVENT_RADIOSEARCH,
+        EVENT_MOVESONGUP,
+        EVENT_MOVESONGDOWN,
+        EVENT_ENQUEUEANDPLAY
 };
 
 struct Event
@@ -50,3 +59,5 @@ typedef struct
         char *seq;
         enum EventType eventType;
 } EventMapping;
+
+#endif
