@@ -1,4 +1,4 @@
-/* kew - a command-line music player
+/* kew - A terminal music player
 Copyright (C) 2022 Ravachol
 
 http://github.com/ravachol/kew
@@ -70,8 +70,6 @@ OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. */
 #define COOLDOWN_MS 500
 #define COOLDOWN2_MS 100
 
-#define NUM_KEY_MAPPINGS 48
-
 #define TMPPIDFILE "/tmp/kew_"
 
 FILE *logFile = NULL;
@@ -79,7 +77,6 @@ struct winsize windowSize;
 char digitsPressed[MAX_SEQ_LEN];
 int digitsPressedCount = 0;
 static unsigned int updateCounter = 0;
-bool gPressed = false;
 bool startFromTop = false;
 int lastNotifiedId = -1;
 bool songWasRemoved = false;
@@ -90,6 +87,7 @@ struct timespec lastInputTime;
 bool exactSearch = false;
 int fuzzySearchThreshold = 2;
 int maxDigitsPressedCount = 9;
+int isNewSearchTerm = false;
 
 void updateLastInputTime(void)
 {
@@ -188,7 +186,7 @@ struct Event processInput()
                         fuzzySearch(getLibrary(), fuzzySearchThreshold);
                         event.type = EVENT_SEARCH;
                 }
-                else if (((strnlen(event.key, sizeof(event.key)) == 1 && event.key[0] != '\033' && event.key[0] != '\n' && event.key[0] != '\t' && event.key[0] != '\r') || strcmp(event.key, " ") == 0 || (unsigned char)event.key[0] >= 0xC0) && strcmp(event.key, "Z") != 0 && strcmp(event.key, "X") != 0 && strcmp(event.key, "C") != 0 && strcmp(event.key, "V") != 0 && strcmp(event.key, "B") != 0)
+                else if (((strnlen(event.key, sizeof(event.key)) == 1 && event.key[0] != '\033' && event.key[0] != '\n' && event.key[0] != '\t' && event.key[0] != '\r') || strcmp(event.key, " ") == 0 || (unsigned char)event.key[0] >= 0xC0) && strcmp(event.key, "Z") != 0 && strcmp(event.key, "X") != 0 && strcmp(event.key, "C") != 0 && strcmp(event.key, "V") != 0 && strcmp(event.key, "B") != 0 && strcmp(event.key, "N") != 0)
                 {
                         addToSearchText(event.key);
                         resetSearchResult();
@@ -197,11 +195,43 @@ struct Event processInput()
                 }
         }
 
+        if (appState.currentView == RADIOSEARCH_VIEW)
+        {
+                if (strcmp(event.key, "\x7F") == 0 || strcmp(event.key, "\x08") == 0)
+                {
+                        removeFromRadioSearchText();
+                        resetRadioSearchResult();
+                        isNewSearchTerm = true;
+                        event.type = EVENT_RADIOSEARCH;
+                }
+                else if (((strnlen(event.key, sizeof(event.key)) == 1 && event.key[0] != '\033' && event.key[0] != '\n' && event.key[0] != '\t' && event.key[0] != '\r') || strcmp(event.key, " ") == 0 || (unsigned char)event.key[0] >= 0xC0) && strcmp(event.key, "Z") != 0 && strcmp(event.key, "X") != 0 && strcmp(event.key, "C") != 0 && strcmp(event.key, "V") != 0 && strcmp(event.key, "B") != 0 && strcmp(event.key, "N") != 0)
+                {
+                        addToRadioSearchText(event.key);
+                        resetRadioSearchResult();
+                        isNewSearchTerm = true;
+                        event.type = EVENT_RADIOSEARCH;
+                }
+                else if (event.key[0] == '\n')
+                {
+                        if (isNewSearchTerm)
+                        {
+                                radioSearch();
+                                event.type = EVENT_RADIOSEARCH;
+                                isNewSearchTerm = false;
+                        }
+                }
+        }
+
+        if (seq[0] == 127)
+        {
+                seq[0] = '\b'; // Treat as Backspace
+        }
+
         // Set event for pressed key
         for (int i = 0; i < NUM_KEY_MAPPINGS; i++)
         {
                 if (keyMappings[i].seq[0] != '\0' &&
-                    ((seq[0] == '\033' && strnlen(seq, MAX_SEQ_LEN) > 1 && strcmp(seq + 1, keyMappings[i].seq) == 0) ||
+                    ((seq[0] == '\033' && strcmp(seq, "\033\n") != 0 && strnlen(seq, MAX_SEQ_LEN) > 1 && strcmp(seq + 1, keyMappings[i].seq) == 0) ||
                      strcmp(seq, keyMappings[i].seq) == 0))
                 {
                         if (event.type == EVENT_SEARCH && keyMappings[i].eventType != EVENT_GOTOSONG)
@@ -209,22 +239,28 @@ struct Event processInput()
                                 break;
                         }
 
+                        if (event.type == EVENT_RADIOSEARCH && keyMappings[i].eventType != EVENT_GOTOSONG)
+                        {
+                                break;
+                        }
+
                         event.type = keyMappings[i].eventType;
                         break;
                 }
-        }
 
-        // Handle gg
-        if (event.key[0] == 'g' && event.type == EVENT_NONE)
-        {
-                if (gPressed)
+                if (strcmp(seq, "\033\n") == 0 && strcmp(keyMappings[i].seq, "^M") == 0) // ALT+ENTER
                 {
-                        event.type = EVENT_GOTOBEGINNINGOFPLAYLIST;
-                        gPressed = false;
+                        event.type = keyMappings[i].eventType;
+                        break;
                 }
-                else
+
+                // Received mouse input instead of keyboard input
+                if (keyMappings[i].seq[0] != '\0' && strnlen(seq, MAX_SEQ_LEN) > 3 && strncmp(seq, "\033[M", 3) == 0 &&
+                    ((strncmp(seq + 1, keyMappings[i].seq, 3) == 0) ||
+                     strncmp(seq, keyMappings[i].seq, 3) == 0))
                 {
-                        gPressed = true;
+                        event.type = keyMappings[i].eventType;
+                        break;
                 }
         }
 
@@ -290,12 +326,6 @@ struct Event processInput()
                 digitsPressedCount = 0;
         }
 
-        // Forget g pressed
-        if (event.key[0] != 'g')
-        {
-                gPressed = false;
-        }
-
         return event;
 }
 
@@ -314,6 +344,7 @@ void setEndOfListReached(AppState *state)
         pthread_mutex_lock(&dataSourceMutex);
         cleanupPlaybackDevice();
         pthread_mutex_unlock(&dataSourceMutex);
+        stopped = true;
         refresh = true;
 }
 
@@ -356,6 +387,9 @@ void determineSongAndNotify(UISettings *ui)
 
         bool isDeleted = determineCurrentSongData(&currentSongData);
 
+        if (currentSongData && currentSong)
+                currentSong->song.duration = currentSongData->duration;
+
         if (lastNotifiedId != currentSong->id)
         {
                 if (!isDeleted)
@@ -467,35 +501,93 @@ int getSongNumber(const char *str)
         return (int)value;
 }
 
+FileSystemEntry *enqueue(AppState *state, FileSystemEntry *entry)
+{
+        FileSystemEntry *firstEnqueuedEntry = NULL;
+
+        if (audioData.restart)
+        {
+                Node *lastSong = findSelectedEntryById(&playlist, lastPlayedId);
+                startFromTop = false;
+
+                if (lastSong == NULL)
+                {
+                        if (playlist.tail != NULL)
+                                lastPlayedId = playlist.tail->id;
+                        else
+                        {
+                                lastPlayedId = -1;
+                                startFromTop = true;
+                        }
+                }
+        }
+
+        pthread_mutex_lock(&(playlist.mutex));
+
+        firstEnqueuedEntry = enqueueSongs(entry, &state->uiState);
+        resetListAfterDequeuingPlayingSong(state);
+
+        pthread_mutex_unlock(&(playlist.mutex));
+
+        return firstEnqueuedEntry;
+}
+
+void enqueueAndPlay(AppState *state)
+{
+        if (state->currentView == LIBRARY_VIEW || state->currentView == SEARCH_VIEW)
+        {
+                if (isRadioPlaying())
+                {
+                        stopRadio();
+                        setEOFReached();
+                }
+        }
+
+        FileSystemEntry *firstEnqueuedEntry = NULL;
+
+        bool wasEmpty = (playlist.count == 0);
+
+        if (state->currentView == LIBRARY_VIEW)
+        {
+                firstEnqueuedEntry = enqueue(state, getCurrentLibEntry());
+        }
+
+        if (state->currentView == SEARCH_VIEW)
+        {
+                FileSystemEntry *entry = getCurrentSearchEntry();
+                firstEnqueuedEntry = enqueue(state, entry);
+
+                setChosenDir(entry);
+        }
+
+        if (firstEnqueuedEntry && !wasEmpty)
+        {
+                Node *song = findPathInPlaylist(firstEnqueuedEntry->fullPath, &playlist);
+
+                loadedNextSong = true;
+
+                nextSongNeedsRebuilding = false;
+
+                unloadSongA(state);
+                unloadSongB(state);
+
+                usingSongDataA = false;
+                audioData.currentFileIndex = 0;
+                loadingdata.loadA = true;
+
+                playbackPlay(&totalPauseSeconds, &pauseSeconds);
+
+                play(song);
+        }
+}
+
 void handleGoToSong(AppState *state)
 {
         bool canGoNext = (currentSong != NULL && currentSong->next != NULL);
 
         if (state->currentView == LIBRARY_VIEW)
         {
-                if (audioData.restart)
-                {
-                        Node *lastSong = findSelectedEntryById(&playlist, lastPlayedId);
-                        startFromTop = false;
-
-                        if (lastSong == NULL)
-                        {
-                                if (playlist.tail != NULL)
-                                        lastPlayedId = playlist.tail->id;
-                                else
-                                {
-                                        lastPlayedId = -1;
-                                        startFromTop = true;
-                                }
-                        }
-                }
-
-                pthread_mutex_lock(&(playlist.mutex));
-
-                enqueueSongs(getCurrentLibEntry(), &state->uiState);
-                resetListAfterDequeuingPlayingSong(state);
-
-                pthread_mutex_unlock(&(playlist.mutex));
+                enqueue(state, getCurrentLibEntry());
         }
         else if (state->currentView == SEARCH_VIEW)
         {
@@ -511,7 +603,11 @@ void handleGoToSong(AppState *state)
 
                 pthread_mutex_unlock(&(playlist.mutex));
         }
-        else
+        else if (state->currentView == RADIOSEARCH_VIEW)
+        {
+                playRadio();
+        }
+        else if (state->currentView == PLAYLIST_VIEW)
         {
                 if (digitsPressedCount == 0)
                 {
@@ -521,6 +617,12 @@ void handleGoToSong(AppState *state)
                         }
                         else
                         {
+                                if (isRadioPlaying())
+                                {
+                                        stopRadio();
+                                        setEOFReached();
+                                }
+
                                 loadedNextSong = true;
 
                                 nextSongNeedsRebuilding = false;
@@ -669,16 +771,19 @@ void handleInput(AppState *state)
                 updateLibrary(settings.path);
                 break;
         case EVENT_SHOWKEYBINDINGS:
-                toggleShowKeyBindings();
+                toggleShowView(KEYBINDINGS_VIEW);
                 break;
         case EVENT_SHOWPLAYLIST:
-                toggleShowPlaylist();
+                toggleShowView(PLAYLIST_VIEW);
                 break;
         case EVENT_SHOWSEARCH:
-                toggleShowSearch();
+                toggleShowView(SEARCH_VIEW);
+                break;
+        case EVENT_SHOWRADIOSEARCH:
+                toggleShowView(RADIOSEARCH_VIEW);
                 break;
         case EVENT_SHOWLIBRARY:
-                toggleShowLibrary();
+                toggleShowView(LIBRARY_VIEW);
                 break;
         case EVENT_NEXTPAGE:
                 flipNextPage();
@@ -693,8 +798,27 @@ void handleInput(AppState *state)
         case EVENT_SHOWTRACK:
                 showTrack();
                 break;
-        case EVENT_TABNEXT:
-                tabNext();
+        case EVENT_NEXTVIEW:
+                switchToNextView();
+                break;
+        case EVENT_PREVVIEW:
+                switchToPreviousView();
+                break;
+        case EVENT_CLEARPLAYLIST:
+                updatePlaylistToPlayingSong();
+                state->uiState.resetPlaylistDisplay = true;
+                break;
+        case EVENT_RADIOSEARCH:
+                refresh = true;
+                break;
+        case EVENT_MOVESONGUP:
+                moveSongUp();
+                break;
+        case EVENT_MOVESONGDOWN:
+                moveSongDown();
+                break;
+        case EVENT_ENQUEUEANDPLAY:
+                enqueueAndPlay(state);
                 break;
         default:
                 fastForwarding = false;
@@ -788,6 +912,14 @@ void loadAudioData(AppState *state)
                         unloadSongA(state);
                         unloadSongB(state);
 
+                        if (isRadioPlaying())
+                        {
+                                stopRadio();
+                                audioData.currentFileIndex = 0;
+                                loadingdata.loadA = true;
+                                usingSongDataA = false;
+                        }
+
                         int res = loadFirst(currentSong, state);
 
                         finishLoading();
@@ -843,7 +975,7 @@ void tryLoadNext()
         }
 }
 
-void prepareNextSong(AppState *state)
+void handleSkipOutOfOrder(void)
 {
         if (!skipOutOfOrder && !isRepeatEnabled())
         {
@@ -853,9 +985,19 @@ void prepareNextSong(AppState *state)
         {
                 skipOutOfOrder = false;
         }
+}
 
-        finishLoading();
+void resetClock(void)
+{
         resetTimeCount();
+        clock_gettime(CLOCK_MONOTONIC, &start_time);
+}
+
+void prepareNextSong(AppState *state)
+{
+        resetClock();
+        handleSkipOutOfOrder();
+        finishLoading();
 
         nextSong = NULL;
         refresh = true;
@@ -868,16 +1010,18 @@ void prepareNextSong(AppState *state)
         if (currentSong == NULL)
         {
                 if (state->uiSettings.quitAfterStopping)
+                {
                         quit();
+                }
                 else
+                {
                         setEndOfListReached(state);
+                }
         }
         else
         {
                 determineSongAndNotify(&(state->uiSettings));
         }
-
-        clock_gettime(CLOCK_MONOTONIC, &start_time);
 }
 
 void handleSkipFromStopped()
@@ -891,6 +1035,43 @@ void handleSkipFromStopped()
         }
 }
 
+void updatePlayerStatus(AppState *state)
+{
+        updatePlayer(&state->uiState);
+
+        reconnectRadioIfNeeded();
+
+        if (playlist.head != NULL)
+        {
+                if ((skipFromStopped || !loadedNextSong || nextSongNeedsRebuilding) && !audioData.endOfListReached)
+                {
+                        loadAudioData(state);
+                }
+
+                if (songHasErrors)
+                        tryLoadNext();
+
+                if (isPlaybackDone())
+                {
+                        updateLastSongSwitchTime();
+                        prepareNextSong(state);
+                        switchAudioImplementation();
+                }
+        }
+        else
+        {
+                setEOFNotReached();
+        }
+}
+
+void processDBusEvents(void)
+{
+        while (g_main_context_pending(global_main_context))
+        {
+                g_main_context_iteration(global_main_context, FALSE);
+        }
+}
+
 gboolean mainloop_callback(gpointer data)
 {
         (void)data;
@@ -902,38 +1083,13 @@ gboolean mainloop_callback(gpointer data)
         updateCounter++;
 
         // Update every other time or if searching (search needs to update often to detect keypresses)
-        if (updateCounter % 2 == 0 || appState.currentView == SEARCH_VIEW)
+        if (updateCounter % 2 == 0 || appState.currentView == SEARCH_VIEW || appState.currentView == RADIOSEARCH_VIEW)
         {
-                // Process GDBus events in the global_main_context
-                while (g_main_context_pending(global_main_context))
-                {
-                        g_main_context_iteration(global_main_context, FALSE);
-                }
+                processDBusEvents();
 
-                updatePlayer(&appState.uiState);
-
-                if (playlist.head != NULL)
-                {
-                        if ((skipFromStopped || !loadedNextSong || nextSongNeedsRebuilding) && !audioData.endOfListReached)
-                        {
-                                loadAudioData(&appState);
-                        }
-
-                        if (songHasErrors)
-                                tryLoadNext();
-
-                        if (isPlaybackDone())
-                        {
-                                updateLastSongSwitchTime();
-                                prepareNextSong(&appState);
-                                switchAudioImplementation();
-                        }
-                }
-                else
-                {
-                        setEOFNotReached();
-                }
+                updatePlayerStatus(&appState);
         }
+
         return TRUE;
 }
 
@@ -1007,21 +1163,23 @@ void initFirstPlay(Node *song, AppState *state)
 void cleanupOnExit()
 {
         pthread_mutex_lock(&dataSourceMutex);
-        resetDecoders();
-        resetVorbisDecoders();
-        resetOpusDecoders();
-#ifdef USE_FAAD
-        resetM4aDecoders();
-#endif
+
+        resetAllDecoders();
+
         if (isContextInitialized)
         {
                 cleanupPlaybackDevice();
                 cleanupAudioContext();
         }
+
+        stopRadio();
+
         emitPlaybackStoppedMpris();
 
         bool noMusicFound = false;
 
+        FileSystemEntry *library = getLibrary();
+
         if (library == NULL || library->children == NULL)
         {
                 noMusicFound = true;
@@ -1039,6 +1197,9 @@ void cleanupOnExit()
         }
 
         freeSearchResults();
+        freeRadioSearchResults();
+        freeCurrentlyPlayingRadioStation();
+        curl_global_cleanup();
         cleanupMpris();
         restoreTerminalMode();
         enableInputBuffering();
@@ -1062,9 +1223,19 @@ void cleanupOnExit()
 #ifdef USE_DBUS
         cleanupDbusConnection();
 #endif
-        resetConsole();
+
+#ifdef DEBUG
+        fclose(logFile);
+#endif
+        if (freopen("/dev/stderr", "w", stderr) == NULL)
+        {
+                perror("freopen error");
+        }
+
+        printf("\n");
         showCursor();
-        fflush(stdout);
+        exitAlternateScreenBuffer();
+        disableMouseButtons();
 
         if (noMusicFound)
         {
@@ -1077,13 +1248,7 @@ void cleanupOnExit()
                 printf("Music not found.\n");
         }
 
-#ifdef DEBUG
-        fclose(logFile);
-#endif
-        if (freopen("/dev/stderr", "w", stderr) == NULL)
-        {
-                perror("freopen error");
-        }
+        fflush(stdout);
 }
 
 void run(AppState *state)
@@ -1148,14 +1313,14 @@ void init(AppState *state)
         userData.songdataADeleted = true;
         userData.songdataBDeleted = true;
         initAudioBuffer();
-        unsigned int seed = (unsigned int) time(NULL);
+        unsigned int seed = (unsigned int)time(NULL);
         srand(seed);
         pthread_mutex_init(&dataSourceMutex, NULL);
         pthread_mutex_init(&switchMutex, NULL);
         pthread_mutex_init(&(loadingdata.mutex), NULL);
         pthread_mutex_init(&(playlist.mutex), NULL);
-        state->uiSettings.nerdFontsEnabled = true;
         createLibrary(&settings, state);
+        curl_global_init(CURL_GLOBAL_DEFAULT);
         fflush(stdout);
 
 #ifdef DEBUG
@@ -1196,6 +1361,7 @@ void playSpecialPlaylist(AppState *state)
 void playAll(AppState *state)
 {
         init(state);
+        FileSystemEntry *library = getLibrary();
         createPlayListFromFileSystemEntry(library, &playlist, MAX_FILES);
         if (playlist.count == 0)
         {
@@ -1208,6 +1374,7 @@ void playAll(AppState *state)
 void playAllAlbums(AppState *state)
 {
         init(state);
+        FileSystemEntry *library = getLibrary();
         addShuffledAlbumsToPlayList(library, &playlist, MAX_FILES);
         if (playlist.count == 0)
         {
@@ -1294,12 +1461,36 @@ void handleOptions(int *argc, char *argv[], UISettings *ui)
                 removeArgElement(argv, idx, argc);
 }
 
+/*
+ * Checks if a process with the given PID is running
+ *
+ * Returns:
+ *  1 if the process is running, 0 otherwise.
+ */
 int isProcessRunning(pid_t pid)
 {
-        char proc_path[64];
-        snprintf(proc_path, sizeof(proc_path), "/proc/%d", pid);
-        struct stat statbuf;
-        return (stat(proc_path, &statbuf) == 0);
+        if (pid <= 0)
+        {
+                return 0; // Invalid PID
+        }
+
+        // Send signal 0 to check if the process exists
+        if (kill(pid, 0) == 0)
+        {
+                return 1; // Process exists
+        }
+
+        // Check errno for detailed status
+        if (errno == ESRCH)
+        {
+                return 0; // No such process
+        }
+        else if (errno == EPERM)
+        {
+                return 1; // Process exists but we don't have permission
+        }
+
+        return 0; // Other errors
 }
 
 // Ensures only a single instance of kew can run at a time for the current user.
@@ -1321,7 +1512,7 @@ void exitIfAlreadyRunning()
                         if (isProcessRunning(pid))
                         {
                                 fprintf(stderr, "An instance of kew is already running. Pid: %d.\n", pid);
-                                exit(EXIT_FAILURE);
+                                exit(1);
                         }
                         else
                         {
@@ -1340,7 +1531,7 @@ void exitIfAlreadyRunning()
         if (pidfile == NULL)
         {
                 perror("Unable to create PID file");
-                exit(EXIT_FAILURE);
+                exit(1);
         }
         fprintf(pidfile, "%d\n", getpid());
         fclose(pidfile);
@@ -1348,13 +1539,10 @@ void exitIfAlreadyRunning()
 
 int directoryExists(const char *path)
 {
-        struct stat info;
-        if (stat(path, &info) != 0)
-        {
-                return 0;
-        }
-        else if (S_ISDIR(info.st_mode))
+        DIR *dir = opendir(path);
+        if (dir != NULL)
         {
+                closedir(dir);
                 return 1;
         }
 
@@ -1373,6 +1561,8 @@ void setMusicPath()
         struct passwd *pw = getpwuid(getuid());
         char *user = NULL;
 
+        clearScreen();
+
         if (pw)
         {
                 user = pw->pw_name;
@@ -1449,7 +1639,7 @@ void setMusicPath()
                 }
                 else
                 {
-                        printf("The entered path does not exist.\n");
+                        printf("The entered path does not exist or is inaccessible.\n");
                         exit(1);
                 }
         }
@@ -1460,20 +1650,21 @@ void setMusicPath()
 
 void initState(AppState *state)
 {
-        state->uiSettings.nerdFontsEnabled = true;
+        state->uiSettings.uiEnabled = true;
+        state->uiSettings.color.r = 125;
+        state->uiSettings.color.g = 125;
+        state->uiSettings.color.b = 125;
         state->uiSettings.visualizerEnabled = true;
         state->uiSettings.coverEnabled = true;
         state->uiSettings.hideLogo = false;
         state->uiSettings.hideHelp = false;
         state->uiSettings.quitAfterStopping = false;
+        state->uiSettings.hideGlimmeringText = false;
         state->uiSettings.coverAnsi = false;
-        state->uiSettings.uiEnabled = true;
         state->uiSettings.visualizerHeight = 5;
+        state->uiSettings.titleDelay = 9;
         state->uiSettings.cacheLibrary = -1;
         state->uiSettings.useConfigColors = false;
-        state->uiSettings.color.r = 125;
-        state->uiSettings.color.g = 125;
-        state->uiSettings.color.b = 125;
         state->uiState.numDirectoryTreeEntries = 0;
         state->uiState.numProgressBars = 35;
         state->uiState.chosenNodeId = 0;
@@ -1486,6 +1677,15 @@ void initState(AppState *state)
         state->uiState.doNotifyMPRISPlaying = false;
         state->uiState.collapseView = false;
         state->tempCache = NULL;
+
+        radioContext.buf.stale = false;
+}
+
+void initializeStateAndSettings(AppState *appState, AppSettings *settings)
+{
+        initState(appState);
+        getConfig(settings, &appState->uiSettings);
+        mapSettingsToKeys(settings, &appState->uiSettings, keyMappings);
 }
 
 int main(int argc, char *argv[])
@@ -1505,9 +1705,9 @@ int main(int argc, char *argv[])
                 exit(0);
         }
 
-        initState(&appState);
-        getConfig(&settings, ui);
-        mapSettingsToKeys(&settings, keyMappings);
+        enableMouseButtons();
+
+        initializeStateAndSettings(&appState, &settings);
 
         if (argc == 3 && (strcmp(argv[1], "path") == 0))
         {
@@ -1516,13 +1716,14 @@ int main(int argc, char *argv[])
                 exit(0);
         }
 
+        enterAlternateScreenBuffer();
+        atexit(cleanupOnExit);
+
         if (settings.path[0] == '\0')
         {
                 setMusicPath();
         }
 
-        atexit(cleanupOnExit);
-
         handleOptions(&argc, argv, ui);
         loadSpecialPlaylist(settings.path);
 
@@ -1538,7 +1739,7 @@ int main(int argc, char *argv[])
         {
                 playAllAlbums(&appState);
         }
-        else if (argc == 2 && strcmp(argv[1], ".") == 0)
+        else if (argc == 2 && strcmp(argv[1], ".") == 0 && specialPlaylist->count != 0)
         {
                 playSpecialPlaylist(&appState);
         }
