@@ -7,12 +7,11 @@ Functions related to mpris implementation.
 
 */
 
-GMainContext *global_main_context = NULL;
-GMainLoop *main_loop;
 guint registration_id;
 guint player_registration_id;
 guint bus_name_id;
 
+#ifndef __APPLE__
 static const gchar *LoopStatus = "None";
 static gdouble Rate = 1.0;
 static gdouble Volume = 0.5;
@@ -32,6 +31,8 @@ void updatePlaybackStatus(const gchar *status)
         g_variant_unref(status_variant);
 }
 
+
+
 const gchar *introspection_xml =
     "<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\" \"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">\n"
     "<node>\n"
@@ -104,6 +105,8 @@ static void handle_raise(GDBusConnection *connection, const gchar *sender,
         (void)parameters;
         (void)invocation;
         (void)user_data;
+
+        g_dbus_method_invocation_return_value(invocation, NULL);
 }
 
 static void handle_quit(GDBusConnection *connection, const gchar *sender,
@@ -187,7 +190,7 @@ static void handle_next(GDBusConnection *connection, const gchar *sender,
         (void)parameters;
         (void)user_data;
 
-        skipToNextSong();
+        skipToNextSong(&appState);
         g_dbus_method_invocation_return_value(invocation, NULL);
 }
 
@@ -204,7 +207,7 @@ static void handle_previous(GDBusConnection *connection, const gchar *sender,
         (void)parameters;
         (void)user_data;
 
-        skipToPrevSong();
+        skipToPrevSong(&appState);
         g_dbus_method_invocation_return_value(invocation, NULL);
 }
 
@@ -223,6 +226,7 @@ static void handle_pause(GDBusConnection *connection, const gchar *sender,
         (void)user_data;
 
         playbackPause(&pause_time);
+        g_dbus_method_invocation_return_value(invocation, NULL);
 }
 
 static void handle_play_pause(GDBusConnection *connection, const gchar *sender,
@@ -275,6 +279,7 @@ static void handle_play(GDBusConnection *connection, const gchar *sender,
         (void)user_data;
 
         playbackPlay(&totalPauseSeconds, &pauseSeconds);
+        g_dbus_method_invocation_return_value(invocation, NULL);
 }
 
 static void handle_seek(GDBusConnection *connection,
@@ -350,7 +355,9 @@ static void handle_set_position(GDBusConnection *connection,
                                                       "Failed to set position for track %s", track_id);
         }
 }
+#endif
 
+#ifndef __APPLE__
 static void handle_method_call(GDBusConnection *connection, const gchar *sender,
                                const gchar *object_path, const gchar *interface_name,
                                const gchar *method_name, GVariant *parameters,
@@ -413,7 +420,9 @@ static void handle_method_call(GDBusConnection *connection, const gchar *sender,
                                                            "No such method");
         }
 }
+#endif
 
+#ifndef __APPLE__
 static void on_bus_name_acquired(GDBusConnection *connection, const gchar *name, gpointer user_data)
 {
         (void)connection;
@@ -768,7 +777,9 @@ static gboolean get_can_control(GDBusConnection *connection, const gchar *sender
         *value = g_variant_new_boolean(CanControl);
         return TRUE;
 }
+#endif
 
+#ifndef __APPLE__
 static GVariant *get_property_callback(GDBusConnection *connection, const gchar *sender,
                                        const gchar *object_path, const gchar *interface_name,
                                        const gchar *property_name, GError **error, gpointer user_data)
@@ -926,7 +937,9 @@ static gboolean set_property_callback(GDBusConnection *connection, const gchar *
                 return FALSE;
         }
 }
+#endif
 
+#ifndef __APPLE__
 // MPRIS MediaPlayer2 interface vtable
 static const GDBusInterfaceVTable media_player_interface_vtable = {
     .method_call = handle_method_call,     // We're using individual method handlers
@@ -950,9 +963,11 @@ static const GDBusInterfaceVTable player_interface_vtable = {
         handle_play,
         handle_seek,
         handle_set_position}};
+#endif
 
 void emitPlaybackStoppedMpris()
 {
+#ifndef __APPLE__
         if (connection)
         {
                 g_dbus_connection_call(connection,
@@ -968,10 +983,12 @@ void emitPlaybackStoppedMpris()
                                        NULL,
                                        NULL);
         }
+#endif
 }
 
-void cleanupMpris()
+void cleanupMpris(void)
 {
+#ifndef __APPLE__
         if (registration_id > 0)
         {
                 g_dbus_connection_unregister_object(connection, registration_id);
@@ -1001,17 +1018,12 @@ void cleanupMpris()
                 g_main_context_unref(global_main_context);
                 global_main_context = NULL;
         }
-
-#ifdef USE_LIBNOTIFY
-        if (previous_notification != NULL) {
-                g_object_unref(previous_notification);
-                previous_notification = NULL;
-        }        
 #endif
 }
 
-void initMpris()
+void initMpris(void)
 {
+#ifndef __APPLE__
         if (global_main_context == NULL)
         {
                 global_main_context = g_main_context_new();
@@ -1079,10 +1091,12 @@ void initMpris()
         }
 
         g_dbus_node_info_unref(introspection_data);
+#endif
 }
 
 void emitStartPlayingMpris()
 {
+#ifndef __APPLE__
         GVariant *parameters = g_variant_new("(s)", "Playing");
         g_dbus_connection_emit_signal(connection,
                                       NULL,
@@ -1091,29 +1105,33 @@ void emitStartPlayingMpris()
                                       "PlaybackStatusChanged",
                                       parameters,
                                       NULL);
+#endif
 }
 
 gchar *sanitizeTitle(const gchar *title)
 {
-    gchar *sanitized = g_strdup(title);
+        gchar *sanitized = g_strdup(title);
 
-    // Replace underscores with hyphens, otherwise some widgets have a problem
-    g_strdelimit(sanitized, "_", '-');
+        // Replace underscores with hyphens, otherwise some widgets have a problem
+        g_strdelimit(sanitized, "_", '-');
 
-    // duplicate string otherwise widgets have a problem with certain strings for some reason
-    gchar *sanitized_dup = g_strdup_printf("%s", sanitized);
+        // duplicate string otherwise widgets have a problem with certain strings for some reason
+        gchar *sanitized_dup = g_strdup_printf("%s", sanitized);
 
-    g_free(sanitized);
+        g_free(sanitized);
 
-    return sanitized_dup;
+        return sanitized_dup;
 }
 
+#ifndef __APPLE__
 static guint64 last_emit_time = 0;
+#endif
 
 void emit_properties_changed(GDBusConnection *connection,
                              const gchar *property_name,
                              GVariant *new_value)
 {
+#ifndef __APPLE__
         GVariantBuilder changed_properties_builder;
 
         if (connection == NULL || property_name == NULL || new_value == NULL)
@@ -1138,10 +1156,16 @@ void emit_properties_changed(GDBusConnection *connection,
         }
 
         g_variant_builder_clear(&changed_properties_builder);
+#else
+        (void)connection;
+        (void)property_name;
+        (void)new_value;
+#endif
 }
 
-void emitVolumeChanged()
+void emitVolumeChanged(void)
 {
+#ifndef __APPLE__
         gdouble newVolume = (gdouble)getCurrentVolume() / 100;
 
         if (newVolume > 1.0)
@@ -1150,19 +1174,23 @@ void emitVolumeChanged()
         // Emit the PropertiesChanged signal for the volume property
         GVariant *volume_variant = g_variant_new_double(newVolume);
         emit_properties_changed(connection, "Volume", volume_variant);
+#endif
 }
 
-void emitShuffleChanged()
+void emitShuffleChanged(void)
 {
+#ifndef __APPLE__
         gboolean shuffleEnabled = isShuffleEnabled();
 
         // Emit the PropertiesChanged signal for the volume property
         GVariant *volume_variant = g_variant_new_boolean(shuffleEnabled);
         emit_properties_changed(connection, "Shuffle", volume_variant);
+#endif
 }
 
 void emitMetadataChanged(const gchar *title, const gchar *artist, const gchar *album, const gchar *coverArtPath, const gchar *trackId, Node *currentSong, gint64 length)
 {
+#ifndef __APPLE__
         guint64 current_time = g_get_monotonic_time();
         if (current_time - last_emit_time < 500000) // 0.5 seconds
         {
@@ -1228,12 +1256,14 @@ void emitMetadataChanged(const gchar *title, const gchar *artist, const gchar *a
         g_variant_builder_add(&changed_properties_builder, "{sv}", "Metadata", metadata_variant);
         g_variant_builder_add(&changed_properties_builder, "{sv}", "CanGoPrevious", g_variant_new_boolean((currentSong != NULL && currentSong->prev != NULL)));
         g_variant_builder_add(&changed_properties_builder, "{sv}", "CanGoNext", g_variant_new_boolean((currentSong != NULL && currentSong->next != NULL)));
+        g_variant_builder_add(&changed_properties_builder, "{sv}", "Shuffle", g_variant_new_boolean(isShuffleEnabled()));
+        g_variant_builder_add(&changed_properties_builder, "{sv}", "CanPlay", g_variant_new_boolean(length != 0 ? true : false));
+        g_variant_builder_add(&changed_properties_builder, "{sv}", "CanPause", g_variant_new_boolean(length != 0 ? true : false));
+        g_variant_builder_add(&changed_properties_builder, "{sv}", "LoopStatus", g_variant_new_string(isRepeatEnabled() ? "Track" : "None"));
+
+
 
         CanSeek = true;
-        if (currentSong != NULL && endsWith(currentSong->song.filePath, "ogg"))
-        {
-                CanSeek = false;
-        }
 
         g_variant_builder_add(&changed_properties_builder, "{sv}", "CanSeek", g_variant_new_boolean(CanSeek));
 
@@ -1255,4 +1285,13 @@ void emitMetadataChanged(const gchar *title, const gchar *artist, const gchar *a
 
         g_variant_builder_clear(&changed_properties_builder);
         g_variant_builder_clear(&metadata_builder);
+#else
+        (void)title;
+        (void)artist;
+        (void)album;
+        (void)coverArtPath;
+        (void)trackId;
+        (void)currentSong;
+        (void)length;
+#endif
 }
