@@ -2,13 +2,21 @@
 #ifndef PLAYEROPS_H
 #define PLAYEROPS_H
 
+#include <dirent.h>
+#include <pthread.h>
+#include <stdio.h>
 #include <stdbool.h>
+#include <stdlib.h>
 #include <sys/time.h>
+#include <unistd.h>
+#include "appstate.h"
 #include "player.h"
 #include "songloader.h"
 #include "settings.h"
 #include "soundcommon.h"
-
+#ifdef USE_FAAD
+#include "m4a.h"
+#endif
 #ifndef CLOCK_MONOTONIC
 #define CLOCK_MONOTONIC 1
 #endif
@@ -28,69 +36,56 @@ typedef struct
 } LoadingThreadData;
 
 extern GDBusConnection *connection;
+extern GMainContext *global_main_context;
 extern LoadingThreadData loadingdata;
-extern double elapsedSeconds;
-extern double pauseSeconds;
-extern double totalPauseSeconds;
+extern struct timespec start_time;
 extern struct timespec pause_time;
 extern volatile bool loadedNextSong;
-extern bool playlistNeedsUpdate;
 extern bool nextSongNeedsRebuilding;
-extern bool enqueuedNeedsUpdate;
 extern bool waitingForPlaylist;
 extern bool waitingForNext;
 extern bool usingSongDataA;
 extern Node *nextSong;
+extern Node *songToStartFrom;
 extern int lastPlayedId;
-extern bool playingMainPlaylist;
 extern bool songHasErrors;
-extern bool doQuit;
-extern bool loadingFailed;
 extern volatile bool clearingErrors;
 extern volatile bool songLoading;
-extern struct timespec start_time;
 extern bool skipping;
 extern bool skipOutOfOrder;
 extern Node *tryNextSong;
-extern struct timespec lastInputTime;
-extern struct timespec lastPlaylistChangeTime;
 extern bool skipFromStopped;
-
 extern UserData userData;
 
 SongData *getCurrentSongData(void);
 
-void rebuildAndUpdatePlaylist();
+Node *getNextSong(void);
 
-Node *getNextSong();
+void handleRemove(void);
 
-void handleRemove();
-
-void enqueueSongs(FileSystemEntry *entry);
+FileSystemEntry *enqueueSongs(FileSystemEntry *entry, UIState *uis);
 
 void updateLastSongSwitchTime(void);
 
-void updateLastInputTime(void);
-
 void playbackPause(struct timespec *pause_time);
 
 void playbackPlay(double *totalPauseSeconds, double *pauseSeconds);
 
 void togglePause(double *totalPauseSeconds, double *pauseSeconds, struct timespec *pause_time);
 
+void stop(void);
+
 void toggleRepeat(void);
 
 void toggleShuffle(void);
 
 void addToSpecialPlaylist(void);
 
-void toggleBlocks(AppSettings *settings);
+void toggleBlocks(AppSettings *settings, UISettings *ui);
 
-void toggleColors(AppSettings *settings);
+void toggleColors(AppSettings *settings, UISettings *ui);
 
-void toggleCovers(AppSettings *settings);
-
-void toggleVisualizer(AppSettings *settings);
+void toggleVisualizer(AppSettings *settings, UISettings *ui);
 
 void quit(void);
 
@@ -98,15 +93,15 @@ void calcElapsedTime(void);
 
 Node *getSongByNumber(PlayList *playlist, int songNumber);
 
-void skipToNextSong(void);
+void skipToNextSong(AppState *state);
 
-void skipToPrevSong(void);
+void skipToPrevSong(AppState *state);
 
-void skipToSong(int id);
+void skipToSong(int id, bool startPlaying);
 
-void seekForward(void);
+void seekForward(UIState *uis);
 
-void seekBack(void);
+void seekBack(UIState *uis);
 
 void skipToNumberedSong(int songNumber);
 
@@ -116,7 +111,7 @@ void loadSong(Node *song, LoadingThreadData *loadingdata);
 
 void loadNext(LoadingThreadData *loadingdata);
 
-int loadFirst(Node *song);
+int loadFirst(Node *song, AppState *state);
 
 void flushSeek(void);
 
@@ -128,10 +123,46 @@ void rebuildNextSong(Node *song);
 
 void updateLibrary(char *path);
 
-void askIfCacheLibrary();
+void askIfCacheLibrary(UISettings *ui);
 
-void unloadPreviousSong();
+void unloadSongA(AppState *state);
 
-void createLibrary(AppSettings *settings);
+void unloadSongB(AppState *state);
+
+void unloadPreviousSong(AppState *state);
+
+void createLibrary(AppSettings *settings, AppState *state);
+
+void loadNextSong(void);
+
+void setCurrentSongToNext(void);
+
+void finishLoading(void);
+
+void resetTimeCount(void);
+
+bool setPosition(gint64 newPosition);
+
+bool seekPosition(gint64 offset);
+
+void silentSwitchToNext(bool loadSong, AppState *state);
+
+void reshufflePlaylist(void);
+
+bool determineCurrentSongData(SongData **currentSongData);
+
+void updateLibraryIfChangedDetected(void);
+
+double getCurrentSongDuration(void);
+
+void updatePlaylistToPlayingSong(void);
+
+void playRadio();
+
+void moveSongUp();
+
+void moveSongDown();
+
+void play(Node *song);
 
 #endif
