@@ -16,7 +16,7 @@ Playlist related functions.
 #define MAXPATHLEN 4096
 #endif
 
-const char PLAYLIST_EXTENSIONS[] = "\\.(m3u)$";
+const char PLAYLIST_EXTENSIONS[] = "(m3u8?)$";
 const char mainPlaylistName[] = "kew.m3u";
 
 // The playlist unshuffled as it appears in playlist view
@@ -67,6 +67,57 @@ void addToList(PlayList *list, Node *newNode)
         }
 }
 
+void moveUpList(PlayList *list, Node *node)
+{
+    if (node == list->head || node == NULL || node->prev == NULL)
+        return;
+
+    Node *prevNode = node->prev;
+    Node *nextNode = node->next;
+
+    if (prevNode->prev)
+        prevNode->prev->next = node;
+    else
+        list->head = node;
+
+    node->prev = prevNode->prev;
+    node->next = prevNode;
+
+    prevNode->prev = node;
+    prevNode->next = nextNode;
+
+    if (nextNode)
+        nextNode->prev = prevNode;
+    else
+        list->tail = prevNode;
+}
+
+void moveDownList(PlayList *list, Node *node)
+{
+    if (node == list->tail || node == NULL || node->next == NULL)
+        return;
+
+    Node *nextNode = node->next;
+    Node *prevNode = node->prev;
+    Node *nextNextNode = nextNode->next;
+
+    if (prevNode)
+        prevNode->next = nextNode;
+    else
+        list->head = nextNode;
+
+    nextNode->prev = prevNode;
+    nextNode->next = node;
+
+    node->prev = nextNode;
+    node->next = nextNextNode;
+
+    if (nextNextNode)
+        nextNextNode->prev = node;
+    else
+        list->tail = node;
+}
+
 Node *deleteFromList(PlayList *list, Node *node)
 {
         if (list->head == NULL || node == NULL)
@@ -210,23 +261,6 @@ void shufflePlaylistStartingFromSong(PlayList *playlist, Node *song)
         }
 }
 
-int compare(const struct dirent **a, const struct dirent **b)
-{
-        const char *nameA = (*a)->d_name;
-        const char *nameB = (*b)->d_name;
-
-        if (nameA[0] == '_' && nameB[0] != '_')
-        {
-                return -1;
-        }
-        else if (nameA[0] != '_' && nameB[0] == '_')
-        {
-                return 1;
-        }
-
-        return strcmp(nameA, nameB);
-}
-
 void createNode(Node **node, const char *directoryPath, int id)
 {
         SongInfo song;
@@ -237,6 +271,7 @@ void createNode(Node **node, const char *directoryPath, int id)
         if (*node == NULL)
         {
                 printf("Failed to allocate memory.");
+                free(song.filePath);
                 exit(0);
                 return;
         }
@@ -274,9 +309,9 @@ void buildPlaylistRecursive(const char *directoryPath, const char *allowedExtens
                 return;
         }
 
-        char exto[6];
+        char exto[100];
         struct dirent **entries;
-        int numEntries = scandir(directoryPath, &entries, NULL, compare);
+        int numEntries = scandir(directoryPath, &entries, NULL, compareLibEntries);
 
         if (numEntries < 0)
         {
@@ -327,41 +362,6 @@ void buildPlaylistRecursive(const char *directoryPath, const char *allowedExtens
         regfree(&regex);
 }
 
-int playDirectory(const char *directoryPath, const char *allowedExtensions, PlayList *playlist)
-{
-        DIR *dir = opendir(directoryPath);
-        if (dir == NULL)
-        {
-                printf("Failed to open directory: %s\n", directoryPath);
-                return -1;
-        }
-
-        regex_t regex;
-        int ret = regcomp(&regex, allowedExtensions, REG_EXTENDED);
-        if (ret != 0)
-        {
-                return -1;
-        }
-        char ext[6];
-        struct dirent *entry;
-        while ((entry = readdir(dir)) != NULL)
-        {
-                extractExtension(entry->d_name, sizeof(ext) - 1, ext);
-                if (match_regex(&regex, ext) == 0)
-                {
-                        char filePath[FILENAME_MAX];
-                        snprintf(filePath, sizeof(filePath), "%s/%s", directoryPath, entry->d_name);
-
-                        Node *node = NULL;
-                        createNode(&node, filePath, nodeIdCounter++);
-                        addToList(playlist, node);
-                }
-        }
-        closedir(dir);
-
-        return 0;
-}
-
 int joinPlaylist(PlayList *dest, PlayList *src)
 {
         if (src->count == 0)
@@ -389,61 +389,58 @@ int joinPlaylist(PlayList *dest, PlayList *src)
         return 1;
 }
 
-void makePlaylistName(const char *search)
+void makePlaylistName(const char *search, int maxSize)
 {
-        char *duplicateSearch = strdup(search);
-        strcat(playlistName, duplicateSearch);
-        free(duplicateSearch);
-        strcat(playlistName, ".m3u");
-        int i = 0;
-        while (playlistName[i] != '\0')
+        playlistName[0] = '\0';
+
+        snprintf(playlistName, maxSize, "%s", search);
+
+        snprintf(playlistName + strnlen(playlistName, maxSize), maxSize - strnlen(playlistName, maxSize), ".m3u");
+
+        for (int i = 0; playlistName[i] != '\0'; i++)
         {
                 if (playlistName[i] == ':')
                 {
                         playlistName[i] = '-';
                 }
-                i++;
         }
 }
 
 void readM3UFile(const char *filename, PlayList *playlist)
 {
-        FILE *file = fopen(filename, "r");
-        char directory[MAXPATHLEN];
+        GError *error = NULL;
+        gchar *contents;
+        gchar **lines;
 
-        if (file == NULL)
+        if (!g_file_get_contents(filename, &contents, NULL, &error))
         {
+                g_clear_error(&error);
                 return;
         }
 
-        getDirectoryFromPath(filename, directory);
-        char line[MAXPATHLEN];
-        while (fgets(line, sizeof(line), file))
+        gchar *directory = g_path_get_dirname(filename);
+
+        lines = g_strsplit(contents, "\n", -1);
+
+        for (gint i = 0; lines[i] != NULL; i++)
         {
-                size_t len = strcspn(line, "\r\n");
-                line[len] = '\0';
+                gchar *line = lines[i];
 
-                size_t start = 0;
-                while (isspace(line[start]))
+                line = g_strdelimit(line, "\r", '\0');
+                gchar *trimmed_line = g_strstrip(line);
+
+                if (trimmed_line[0] != '#' && trimmed_line[0] != '\0')
                 {
-                        start++;
-                }
-                size_t end = strlen(line);
-                while (end > start && isspace(line[end - 1]))
-                {
-                        end--;
-                }
-                line[end] = '\0';
+                        gchar *songPath;
 
-                if (line[0] != '#' && line[0] != '\0')
-                {
-                        char songPath[MAXPATHLEN];
-                        memset(songPath, '\0', sizeof(songPath));
-
-                        if (strchr(line, '/') == NULL && strchr(line, '\\') == NULL)
-                                strcat(songPath, directory);
-
-                        strcat(songPath, line);
+                        if (g_path_is_absolute(trimmed_line))
+                        {
+                                songPath = g_strdup(trimmed_line);
+                        }
+                        else
+                        {
+                                songPath = g_build_filename(directory, trimmed_line, NULL);
+                        }
 
                         Node *newNode = NULL;
                         createNode(&newNode, songPath, nodeIdCounter++);
@@ -461,9 +458,14 @@ void readM3UFile(const char *filename, PlayList *playlist)
                         }
 
                         playlist->count++;
+
+                        g_free(songPath);
                 }
         }
-        fclose(file);
+
+        g_free(directory);
+        g_strfreev(lines);
+        g_free(contents);
 }
 
 int makePlaylist(int argc, char *argv[], bool exactSearch, const char *path)
@@ -518,10 +520,10 @@ int makePlaylist(int argc, char *argv[], bool exactSearch, const char *path)
 
         for (int i = start - 1; i < argc; i++)
         {
-                strcat(search, " ");
-                strcat(search, argv[i]);
+                size_t len = strnlen(search, MAX_SEARCH_SIZE);
+                snprintf(search + len, MAX_SEARCH_SIZE - len, " %s", argv[i]);
         }
-        makePlaylistName(search);
+        makePlaylistName(search, MAX_SEARCH_SIZE);
 
         if (strstr(search, delimiter))
         {
@@ -545,11 +547,13 @@ int makePlaylist(int argc, char *argv[], bool exactSearch, const char *path)
                         char buf[MAXPATHLEN] = {0};
                         if (strncmp(token, "song", 4) == 0)
                         {
-                                memmove(token, token + 4, strlen(token + 4) + 1);
+                                memmove(token, token + 4, strnlen(token + 4, MAXPATHLEN) + 1);
                                 searchType = FileOnly;
                         }
-                        trim(token);
-                        if (walker(path, token, buf, allowedExtensions, searchType, exactSearch) == 0)
+                        trim(token, MAXPATHLEN);
+                        char *searching = g_utf8_casefold(token, -1);
+
+                        if (walker(path, searching, buf, allowedExtensions, searchType, exactSearch) == 0)
                         {
                                 if (strcmp(argv[1], "list") == 0)
                                 {
@@ -565,7 +569,7 @@ int makePlaylist(int argc, char *argv[], bool exactSearch, const char *path)
                                         pthread_mutex_unlock(&(playlist.mutex));
                                 }
                         }
-
+                        free(searching);
                         token = strtok(NULL, delimiter);
                 }
         }
@@ -580,34 +584,47 @@ int makePlaylist(int argc, char *argv[], bool exactSearch, const char *path)
         return 0;
 }
 
+void generateM3UFilename(const char *basePath, const char *filePath, char *m3uFilename, size_t size)
+{
 
-void generateM3UFilename(const char *basePath, const char *filePath, char *m3uFilename, size_t size) {
-
-    const char *baseName = strrchr(filePath, '/');
-    if (baseName == NULL) {
-        baseName = filePath; // No '/' found, use the entire filename
-    } else {
-        baseName++; // Skip the '/' character
-    }
-
-    const char *dot = strrchr(baseName, '.');
-    if (dot == NULL) {
-        // No '.' found, copy the base name and append ".m3u"
-        if (basePath[strlen(basePath) - 1] == '/') {
-            snprintf(m3uFilename, size, "%s%s.m3u", basePath, baseName);
-        } else {
-            snprintf(m3uFilename, size, "%s/%s.m3u", basePath, baseName);
+        const char *baseName = strrchr(filePath, '/');
+        if (baseName == NULL)
+        {
+                baseName = filePath; // No '/' found, use the entire filename
         }
-    } else {
-        // Copy the base name up to the dot and append ".m3u"
-        size_t baseNameLen = dot - baseName;
-        if (basePath[strlen(basePath) - 1] == '/') {
-            snprintf(m3uFilename, size, "%s%.*s.m3u", basePath, (int)baseNameLen, baseName);
-        } else {
-            snprintf(m3uFilename, size, "%s/%.*s.m3u", basePath, (int)baseNameLen, baseName);
+        else
+        {
+                baseName++; // Skip the '/' character
+        }
+
+        const char *dot = strrchr(baseName, '.');
+        if (dot == NULL)
+        {
+                // No '.' found, copy the base name and append ".m3u"
+                if (basePath[strnlen(basePath, MAXPATHLEN) - 1] == '/')
+                {
+                        snprintf(m3uFilename, size, "%s%s.m3u", basePath, baseName);
+                }
+                else
+                {
+                        snprintf(m3uFilename, size, "%s/%s.m3u", basePath, baseName);
+                }
+        }
+        else
+        {
+                // Copy the base name up to the dot and append ".m3u"
+                size_t baseNameLen = dot - baseName;
+                if (basePath[strnlen(basePath, MAXPATHLEN) - 1] == '/')
+                {
+                        snprintf(m3uFilename, size, "%s%.*s.m3u", basePath, (int)baseNameLen, baseName);
+                }
+                else
+                {
+                        snprintf(m3uFilename, size, "%s/%.*s.m3u", basePath, (int)baseNameLen, baseName);
+                }
         }
-    }
 }
+
 void writeM3UFile(const char *filename, PlayList *playlist)
 {
         FILE *file = fopen(filename, "w");
@@ -628,10 +645,16 @@ void writeM3UFile(const char *filename, PlayList *playlist)
 void loadSpecialPlaylist(const char *directory)
 {
         char playlistPath[MAXPATHLEN];
-        c_strcpy(playlistPath, sizeof(playlistPath), directory);
-        if (playlistPath[strlen(playlistPath) - 1] != '/')
-                strcat(playlistPath, "/");
-        strcat(playlistPath, mainPlaylistName);
+
+        if (directory[strnlen(directory, MAXPATHLEN) - 1] == '/')
+        {
+                snprintf(playlistPath, sizeof(playlistPath), "%s%s", directory, mainPlaylistName);
+        }
+        else
+        {
+                snprintf(playlistPath, sizeof(playlistPath), "%s/%s", directory, mainPlaylistName);
+        }
+
         specialPlaylist = malloc(sizeof(PlayList));
         if (specialPlaylist == NULL)
         {
@@ -646,28 +669,33 @@ void loadSpecialPlaylist(const char *directory)
 
 void saveSpecialPlaylist(const char *directory)
 {
-        if (directory == NULL)
-        {
-                return;
-        }
+    if (directory == NULL)
+    {
+        return;
+    }
 
-        char playlistPath[MAXPATHLEN];
-        playlistPath[0] = '\0';
+    char playlistPath[MAXPATHLEN];
 
-        c_strcpy(playlistPath, sizeof(playlistPath), directory);
+    int length = snprintf(playlistPath, sizeof(playlistPath), "%s", directory);
 
-        if (playlistPath[0] == '\0')
-        {
-                return;
-        }
+    if (length < 0 || length >= (int)sizeof(playlistPath) || playlistPath[0] == '\0')
+    {
+        return;
+    }
 
-        if (playlistPath[strlen(playlistPath) - 1] != '/')
-                strcat(playlistPath, "/");
+    if (playlistPath[length - 1] != '/')
+    {
+        snprintf(playlistPath + length, sizeof(playlistPath) - length, "/%s", mainPlaylistName);
+    }
+    else
+    {
+        snprintf(playlistPath + length, sizeof(playlistPath) - length, "%s", mainPlaylistName);
+    }
 
-        strcat(playlistPath, mainPlaylistName);
-
-        if (specialPlaylist != NULL && specialPlaylist->count > 0)
-                writeM3UFile(playlistPath, specialPlaylist);
+    if (specialPlaylist != NULL && specialPlaylist->count > 0)
+    {
+        writeM3UFile(playlistPath, specialPlaylist);
+    }
 }
 
 void savePlaylist(const char *path)
@@ -679,7 +707,7 @@ void savePlaylist(const char *path)
 
         if (playlist.head == NULL || playlist.head->song.filePath == NULL)
                 return;
-        
+
         char m3uFilename[MAXPATHLEN];
 
         generateM3UFilename(path, playlist.head->song.filePath, m3uFilename, sizeof(m3uFilename));
@@ -749,7 +777,7 @@ void deepCopyPlayListOntoList(PlayList *originalList, PlayList *newList)
         newList->count = originalList->count;
 }
 
-Node *findPathInPlaylist(char *path, PlayList *playlist)
+Node *findPathInPlaylist(const char *path, PlayList *playlist)
 {
         Node *currentNode = playlist->head;
 
@@ -765,7 +793,7 @@ Node *findPathInPlaylist(char *path, PlayList *playlist)
         return NULL;
 }
 
-Node *findLastPathInPlaylist(char *path, PlayList *playlist)
+Node *findLastPathInPlaylist(const char *path, PlayList *playlist)
 {
         Node *currentNode = playlist->tail;
 
@@ -841,83 +869,84 @@ void createPlayListFromFileSystemEntry(FileSystemEntry *root, PlayList *list, in
 
 int isMusicFile(const char *filename)
 {
-    const char *extensions[] = {".m4a", ".aac", ".mp3", ".ogg", ".flac",".wav", ".opus"};
+        const char *extensions[] = {".m4a", ".aac", ".mp3", ".ogg", ".flac", ".wav", ".opus"};
 
-    size_t numExtensions = sizeof(extensions) / sizeof(extensions[0]);
+        size_t numExtensions = sizeof(extensions) / sizeof(extensions[0]);
 
-    for (size_t i = 0; i < numExtensions; i++)
-    {
-        if (strstr(filename, extensions[i]) != NULL)
+        for (size_t i = 0; i < numExtensions; i++)
         {
-            return 1;
+                if (strstr(filename, extensions[i]) != NULL)
+                {
+                        return 1;
+                }
         }
-    }
 
-    return 0;
+        return 0;
 }
 
 int containsMusicFiles(FileSystemEntry *entry)
 {
-    if (entry == NULL)
-        return 0;
+        if (entry == NULL)
+                return 0;
 
-    FileSystemEntry *child = entry->children;
+        FileSystemEntry *child = entry->children;
 
-    while (child != NULL)
-    {
-        if (!child->isDirectory && isMusicFile(child->name))
+        while (child != NULL)
         {
-            return 1;
+                if (!child->isDirectory && isMusicFile(child->name))
+                {
+                        return 1;
+                }
+                child = child->next;
         }
-        child = child->next;
-    }
 
-    return 0;
+        return 0;
 }
 
 void addAlbumToPlayList(PlayList *list, FileSystemEntry *album, int playlistMax)
 {
-    FileSystemEntry *entry = album->children;
+        FileSystemEntry *entry = album->children;
 
-    while (entry != NULL && list->count < playlistMax)
-    {
-        if (!entry->isDirectory && isMusicFile(entry->name))
+        while (entry != NULL && list->count < playlistMax)
         {
-            addSongToPlayList(list, entry->fullPath, playlistMax);
+                if (!entry->isDirectory && isMusicFile(entry->name))
+                {
+                        addSongToPlayList(list, entry->fullPath, playlistMax);
+                }
+                entry = entry->next;
         }
-        entry = entry->next;
-    }
 }
 
 void addAlbumsToPlayList(FileSystemEntry *entry, PlayList *list, int playlistMax)
 {
-    if (entry == NULL || list->count >= playlistMax)
-        return;
+        if (entry == NULL || list->count >= playlistMax)
+                return;
 
-    if (entry->isDirectory && containsMusicFiles(entry))
-    {
-        addAlbumToPlayList(list, entry, playlistMax);
-    }
+        if (entry->isDirectory && containsMusicFiles(entry))
+        {
+                addAlbumToPlayList(list, entry, playlistMax);
+        }
 
-    if (entry->isDirectory && entry->children != NULL)
-    {
-        addAlbumsToPlayList(entry->children, list, playlistMax);
-    }
+        if (entry->isDirectory && entry->children != NULL)
+        {
+                addAlbumsToPlayList(entry->children, list, playlistMax);
+        }
 
-    if (entry->next != NULL)
-    {
-        addAlbumsToPlayList(entry->next, list, playlistMax);
-    }
+        if (entry->next != NULL)
+        {
+                addAlbumsToPlayList(entry->next, list, playlistMax);
+        }
 }
 
 void shuffleEntries(FileSystemEntry **array, size_t n)
 {
     if (n > 1)
     {
-        size_t i;
-        for (i = 0; i < n - 1; i++)
+        for (size_t i = 0; i < n - 1; i++)
         {
-            size_t j = i + rand() / (RAND_MAX / (n - i) + 1);
+            size_t j = getRandomNumber(i, n - 1);
+
+            // Swap entries at i and j
             FileSystemEntry *temp = array[i];
             array[i] = array[j];
             array[j] = temp;
@@ -927,39 +956,38 @@ void shuffleEntries(FileSystemEntry **array, size_t n)
 
 void collectAlbums(FileSystemEntry *entry, FileSystemEntry **albums, size_t *count)
 {
-    if (entry == NULL)
-        return;
+        if (entry == NULL)
+                return;
 
-    if (entry->isDirectory && containsMusicFiles(entry))
-    {
-        albums[*count] = entry;
-        (*count)++;
-    }
+        if (entry->isDirectory && containsMusicFiles(entry))
+        {
+                albums[*count] = entry;
+                (*count)++;
+        }
 
-    if (entry->isDirectory && entry->children != NULL)
-    {
-        collectAlbums(entry->children, albums, count);
-    }
+        if (entry->isDirectory && entry->children != NULL)
+        {
+                collectAlbums(entry->children, albums, count);
+        }
 
-    if (entry->next != NULL)
-    {
-        collectAlbums(entry->next, albums, count);
-    }
+        if (entry->next != NULL)
+        {
+                collectAlbums(entry->next, albums, count);
+        }
 }
 
 void addShuffledAlbumsToPlayList(FileSystemEntry *root, PlayList *list, int playlistMax)
 {
-    size_t maxAlbums = 2000; 
-    FileSystemEntry *albums[maxAlbums];
-    size_t albumCount = 0;
+        size_t maxAlbums = 2000;
+        FileSystemEntry *albums[maxAlbums];
+        size_t albumCount = 0;
 
-    collectAlbums(root, albums, &albumCount);
+        collectAlbums(root, albums, &albumCount);
 
-    srand(time(NULL));
-    shuffleEntries(albums, albumCount);
+        shuffleEntries(albums, albumCount);
 
-    for (size_t i = 0; i < albumCount && list->count < playlistMax; i++)
-    {
-        addAlbumToPlayList(list, albums[i], playlistMax);
-    }
+        for (size_t i = 0; i < albumCount && list->count < playlistMax; i++)
+        {
+                addAlbumToPlayList(list, albums[i], playlistMax);
+        }
 }
