@@ -1,11 +1,14 @@
 #include "playlist_ui.h"
 
-int startIter = 0;
+/*
 
-void getTerminalSize(int *width, int *height)
-{
-        getTermSize(width, height);
-}
+playlist_ui.c
+
+ Playlist UI functions.
+
+*/
+
+int startIter = 0;
 
 Node *determineStartNode(Node *head, int *foundAt, bool *startFromCurrent, int listSize)
 {
@@ -39,7 +42,7 @@ void preparePlaylistString(Node *node, char *buffer, int bufferSize, int shorten
         }
 
         char filePath[MAXPATHLEN];
-        c_strcpy(filePath, sizeof(filePath), node->song.filePath);
+        c_strcpy(filePath, node->song.filePath, sizeof(filePath));
         char *lastSlash = strrchr(filePath, '/');
         char *lastDot = strrchr(filePath, '.');
 
@@ -48,11 +51,11 @@ void preparePlaylistString(Node *node, char *buffer, int bufferSize, int shorten
                 int nameLength = lastDot - lastSlash - 1;
                 nameLength = (nameLength < bufferSize - 1) ? nameLength : bufferSize - 1;
 
-                strncpy(buffer, lastSlash + 1, nameLength);
+                c_strcpy(buffer, lastSlash + 1, nameLength + 1);
                 buffer[nameLength] = '\0';
-                removeUnneededChars(buffer);
+                removeUnneededChars(buffer, nameLength);
                 shortenString(buffer, shortenAmount);
-                trim(buffer);
+                trim(buffer, MAXPATHLEN);
         }
         else
         {
@@ -60,7 +63,7 @@ void preparePlaylistString(Node *node, char *buffer, int bufferSize, int shorten
         }
 }
 
-int displayPlaylistItems(Node *startNode, int startIter, int maxListSize, int termWidth, int indent, int chosenSong, int *chosenNodeId)
+int displayPlaylistItems(Node *startNode, int startIter, int maxListSize, int termWidth, int indent, int chosenSong, int *chosenNodeId, UISettings *ui)
 {
         int numPrintedRows = 0;
         Node *node = startNode;
@@ -74,17 +77,17 @@ int displayPlaylistItems(Node *startNode, int startIter, int maxListSize, int te
 
         for (int i = startIter; node != NULL && i < startIter + maxListSize; i++)
         {
-                preparePlaylistString(node, buffer, MAXPATHLEN, termWidth - indent - 10);
+                preparePlaylistString(node, buffer, MAXPATHLEN, termWidth - indent - 12);
                 if (buffer[0] != '\0')
                 {
-                        if (useProfileColors)
-                                setTextColor(artistColor);
+                        if (ui->useConfigColors)
+                                setTextColor(ui->artistColor);
                         else
-                                setColor();
+                                setColor(ui);
 
                         printBlankSpaces(indent);
 
-                        printf(" %d. ", i + 1);
+                        printf("   %d. ", i + 1);
 
                         setDefaultTextColor();
 
@@ -95,15 +98,16 @@ int displayPlaylistItems(Node *startNode, int startIter, int maxListSize, int te
                                 printf("\x1b[7m");
                         }
 
-                        if (currentSong != NULL && currentSong->id == node->id)
-                        {
-                                printf("\e[1m\e[39m");
-                        }
-
                         if (i + 1 < 10)
                                 printf(" ");
 
-                        printf("%s \n", buffer);
+
+                        if (currentSong != NULL && currentSong->id == node->id)
+                        {
+                                printf("\e[4m\e[1m");
+                        }
+
+                        printf("%s\n", buffer);
 
                         numPrintedRows++;
                 }
@@ -115,10 +119,12 @@ int displayPlaylistItems(Node *startNode, int startIter, int maxListSize, int te
         return numPrintedRows;
 }
 
-int displayPlaylist(PlayList *list, int maxListSize, int indent, int *chosenSong, int *chosenNodeId, bool reset)
+int displayPlaylist(PlayList *list, int maxListSize, int indent, int *chosenSong, int *chosenNodeId, bool reset, AppState *state)
 {
         int termWidth, termHeight;
-        getTerminalSize(&termWidth, &termHeight);
+        getTermSize(&termWidth, &termHeight);
+
+        UISettings *ui = &(state->uiSettings);
 
         int foundAt = -1;
         bool startFromCurrent = false;
@@ -169,15 +175,12 @@ int displayPlaylist(PlayList *list, int maxListSize, int indent, int *chosenSong
                         startNode = startNode->next;
         }
 
-        int printedRows = displayPlaylistItems(startNode, startIter, maxListSize, termWidth, indent, *chosenSong, chosenNodeId);
+        int printedRows = displayPlaylistItems(startNode, startIter, maxListSize, termWidth, indent, *chosenSong, chosenNodeId, ui);
 
-        if (printedRows > 1)
+        while (printedRows < maxListSize)
         {
-                while (printedRows < maxListSize)
-                {
-                        printf("\n");
-                        printedRows++;
-                }
+                printf("\n");
+                printedRows++;
         }
 
         return printedRows;
