@@ -36,6 +36,8 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         c_strcpy(settings.coverEnabled, "1", sizeof(settings.coverEnabled));
         c_strcpy(settings.allowNotifications, "1", sizeof(settings.allowNotifications));
         c_strcpy(settings.coverAnsi, "0", sizeof(settings.coverAnsi));
+        c_strcpy(settings.quitAfterStopping, "0", sizeof(settings.quitAfterStopping));
+        c_strcpy(settings.hideGlimmeringText, "0", sizeof(settings.hideGlimmeringText));
 #ifdef __APPLE__
         c_strcpy(settings.visualizerEnabled, "0", sizeof(settings.visualizerEnabled)); // visualizer looks wonky in default terminal
         c_strcpy(settings.useConfigColors, "1", sizeof(settings.useConfigColors));     // colors from album look wrong in default terminal
@@ -46,8 +48,11 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         c_strcpy(settings.hideLogo, "0", sizeof(settings.hideLogo));
         c_strcpy(settings.hideHelp, "0", sizeof(settings.hideHelp));
         c_strcpy(settings.cacheLibrary, "-1", sizeof(settings.cacheLibrary));
+        c_strcpy(settings.visualizerHeight, "5", sizeof(settings.visualizerHeight));
+        c_strcpy(settings.titleDelay, "9", sizeof(settings.titleDelay));
 
-        c_strcpy(settings.tabNext, "\t", sizeof(settings.tabNext));
+        c_strcpy(settings.nextView, "\t", sizeof(settings.nextView));
+        c_strcpy(settings.prevView, "[Z", sizeof(settings.prevView));
         c_strcpy(settings.volumeUp, "+", sizeof(settings.volumeUp));
         c_strcpy(settings.volumeUpAlt, "=", sizeof(settings.volumeUpAlt));
         c_strcpy(settings.volumeDown, "-", sizeof(settings.volumeDown));
@@ -75,30 +80,51 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         c_strcpy(settings.hardShowPlaylist, "OQ", sizeof(settings.hardShowPlaylist));
         c_strcpy(settings.hardShowPlaylistAlt, "[[B", sizeof(settings.hardShowPlaylistAlt));
         c_strcpy(settings.showPlaylistAlt, "Z", sizeof(settings.showPlaylistAlt));
-        c_strcpy(settings.hardShowKeys, "[17~", sizeof(settings.hardShowKeys));
-        c_strcpy(settings.hardShowKeysAlt, "[17~", sizeof(settings.hardShowKeysAlt));
-        c_strcpy(settings.showKeysAlt, "B", sizeof(settings.showKeysAlt));
+        c_strcpy(settings.hardShowKeys, "[18~", sizeof(settings.hardShowKeys));
+        c_strcpy(settings.hardShowKeysAlt, "[18~", sizeof(settings.hardShowKeysAlt));
+        c_strcpy(settings.showKeysAlt, "N", sizeof(settings.showKeysAlt));
         c_strcpy(settings.hardShowTrack, "OS", sizeof(settings.hardShowTrack));
         c_strcpy(settings.hardShowTrackAlt, "[[D", sizeof(settings.hardShowTrackAlt));
         c_strcpy(settings.showTrackAlt, "C", sizeof(settings.showTrackAlt));
-        c_strcpy(settings.hardEndOfPlaylist, "G", sizeof(settings.hardEndOfPlaylist));
         c_strcpy(settings.hardShowLibrary, "OR", sizeof(settings.hardShowLibrary));
         c_strcpy(settings.hardShowLibraryAlt, "[[C", sizeof(settings.hardShowLibraryAlt));
         c_strcpy(settings.showLibraryAlt, "X", sizeof(settings.showLibraryAlt));
         c_strcpy(settings.hardShowSearch, "[15~", sizeof(settings.hardShowSearch));
         c_strcpy(settings.hardShowSearchAlt, "[[E", sizeof(settings.hardShowSearchAlt));
+        c_strcpy(settings.hardShowRadioSearch, "[17~", sizeof(settings.hardShowSearch));
+        c_strcpy(settings.hardShowRadioSearchAlt, "B", sizeof(settings.hardShowSearchAlt));
         c_strcpy(settings.showSearchAlt, "V", sizeof(settings.showSearchAlt));
+        c_strcpy(settings.showRadioSearchAlt, "B", sizeof(settings.showSearchAlt));
         c_strcpy(settings.hardNextPage, "[6~", sizeof(settings.hardNextPage));
         c_strcpy(settings.hardPrevPage, "[5~", sizeof(settings.hardPrevPage));
         c_strcpy(settings.hardRemove, "[3~", sizeof(settings.hardRemove));
         c_strcpy(settings.hardRemove2, "[P", sizeof(settings.hardRemove2));
+        c_strcpy(settings.mouseLeftClick, "[M ", sizeof(settings.mouseLeftClick));
+        c_strcpy(settings.mouseMiddleClick, "[M!", sizeof(settings.mouseMiddleClick));
+        c_strcpy(settings.mouseRightClick, "[M\"", sizeof(settings.mouseRightClick));
+        c_strcpy(settings.mouseScrollUp, "[M`", sizeof(settings.mouseScrollUp));
+        c_strcpy(settings.mouseScrollDown, "[Ma", sizeof(settings.mouseScrollDown));
+        c_strcpy(settings.mouseAltScrollUp, "[Mh", sizeof(settings.mouseAltScrollUp));
+        c_strcpy(settings.mouseAltScrollDown, "[Mi", sizeof(settings.mouseAltScrollDown));
         c_strcpy(settings.lastVolume, "100", sizeof(settings.lastVolume));
         c_strcpy(settings.color, "6", sizeof(settings.color));
         c_strcpy(settings.artistColor, "6", sizeof(settings.artistColor));
         c_strcpy(settings.titleColor, "6", sizeof(settings.titleColor));
         c_strcpy(settings.enqueuedColor, "6", sizeof(settings.enqueuedColor));
+        c_strcpy(settings.mouseLeftClickAction, "0", sizeof(settings.mouseLeftClickAction));
+        c_strcpy(settings.mouseMiddleClickAction, "1", sizeof(settings.mouseMiddleClickAction));
+        c_strcpy(settings.mouseRightClickAction, "2", sizeof(settings.mouseRightClickAction));
+        c_strcpy(settings.mouseScrollUpAction, "3", sizeof(settings.mouseScrollUpAction));
+        c_strcpy(settings.mouseScrollDownAction, "4", sizeof(settings.mouseScrollDownAction));
+        c_strcpy(settings.mouseAltScrollUpAction, "7", sizeof(settings.mouseAltScrollUpAction));
+        c_strcpy(settings.mouseAltScrollDownAction, "8", sizeof(settings.mouseAltScrollDownAction));
+        c_strcpy(settings.moveSongUp, "t", sizeof(settings.moveSongUp));
+        c_strcpy(settings.moveSongDown, "g", sizeof(settings.moveSongDown));
+        c_strcpy(settings.enqueueAndPlay, "^M", sizeof(settings.enqueueAndPlay));
         c_strcpy(settings.quit, "q", sizeof(settings.quit));
         c_strcpy(settings.hardQuit, "\x1B", sizeof(settings.hardQuit));
+        c_strcpy(settings.hardClearPlaylist, "\b", sizeof(settings.hardClearPlaylist));
+
 
         if (pairs == NULL)
         {
@@ -135,6 +161,10 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                 {
                         snprintf(settings.visualizerHeight, sizeof(settings.visualizerHeight), "%s", pair->value);
                 }
+                else if (strcmp(lowercaseKey, "titledelay") == 0)
+                {
+                        snprintf(settings.titleDelay, sizeof(settings.titleDelay), "%s", pair->value);
+                }
                 else if (strcmp(lowercaseKey, "volumeup") == 0)
                 {
                         snprintf(settings.volumeUp, sizeof(settings.volumeUp), "%s", pair->value);
@@ -231,6 +261,34 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                 {
                         snprintf(settings.titleColor, sizeof(settings.titleColor), "%s", pair->value);
                 }
+                else if (strcmp(lowercaseKey, "mouseleftclickaction") == 0)
+                {
+                        snprintf(settings.mouseLeftClickAction, sizeof(settings.mouseLeftClickAction), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "mousemiddleclickaction") == 0)
+                {
+                        snprintf(settings.mouseMiddleClickAction, sizeof(settings.mouseMiddleClickAction), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "mouserightclickaction") == 0)
+                {
+                        snprintf(settings.mouseRightClickAction, sizeof(settings.mouseRightClickAction), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "mousescrollupaction") == 0)
+                {
+                        snprintf(settings.mouseScrollUpAction, sizeof(settings.mouseScrollUpAction), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "mousescrolldownaction") == 0)
+                {
+                        snprintf(settings.mouseScrollDownAction, sizeof(settings.mouseScrollDownAction), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "mouseshiftscrollupaction") == 0)
+                {
+                        snprintf(settings.mouseAltScrollUpAction, sizeof(settings.mouseAltScrollUpAction), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "mouseshiftscrolldownaction") == 0)
+                {
+                        snprintf(settings.mouseAltScrollDownAction, sizeof(settings.mouseAltScrollDownAction), "%s", pair->value);
+                }
                 else if (strcmp(lowercaseKey, "hidelogo") == 0)
                 {
                         snprintf(settings.hideLogo, sizeof(settings.hideLogo), "%s", pair->value);
@@ -243,6 +301,14 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                 {
                         snprintf(settings.cacheLibrary, sizeof(settings.cacheLibrary), "%s", pair->value);
                 }
+                else if (strcmp(lowercaseKey, "quitonstop") == 0)
+                {
+                        snprintf(settings.quitAfterStopping, sizeof(settings.quitAfterStopping), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "hideglimmeringtext") == 0)
+                {
+                        snprintf(settings.hideGlimmeringText, sizeof(settings.hideGlimmeringText), "%s", pair->value);
+                }
                 else if (strcmp(lowercaseKey, "quit") == 0)
                 {
                         snprintf(settings.quit, sizeof(settings.quit), "%s", pair->value);
@@ -271,8 +337,26 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                         if (strcmp(pair->value, "") != 0)
                                 snprintf(settings.showSearchAlt, sizeof(settings.showSearchAlt), "%s", pair->value);
                 }
-                else if (strcmp(lowercaseKey, "showkeysalt") == 0)
+                else if (strcmp(lowercaseKey, "movesongup") == 0)
                 {
+                        if (strcmp(pair->value, "") != 0)
+                                snprintf(settings.moveSongUp, sizeof(settings.moveSongUp), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "movesongdown") == 0)
+                {
+                        if (strcmp(pair->value, "") != 0)
+                                snprintf(settings.moveSongDown, sizeof(settings.moveSongDown), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "enqueueandplay") == 0)
+                {
+                        if (strcmp(pair->value, "") != 0)
+                                snprintf(settings.enqueueAndPlay, sizeof(settings.enqueueAndPlay), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "showkeysalt") == 0 && strcmp(pair->value, "B") != 0)
+                {
+                        // We need to prevent the previous key B or else config files wont get updated
+                        // to the new key N and B for radio search on macOS
+
                         if (strcmp(pair->value, "") != 0)
                                 snprintf(settings.showKeysAlt, sizeof(settings.showKeysAlt), "%s", pair->value);
                 }
@@ -366,7 +450,7 @@ int getMusicLibraryPath(char *path)
         return 0;
 }
 
-void mapSettingsToKeys(AppSettings *settings, EventMapping *mappings)
+void mapSettingsToKeys(AppSettings *settings, UISettings *ui, EventMapping *mappings)
 {
         mappings[0] = (EventMapping){settings->scrollUpAlt, EVENT_SCROLLPREV};
         mappings[1] = (EventMapping){settings->scrollDownAlt, EVENT_SCROLLNEXT};
@@ -401,21 +485,35 @@ void mapSettingsToKeys(AppSettings *settings, EventMapping *mappings)
         mappings[30] = (EventMapping){settings->hardShowKeys, EVENT_SHOWKEYBINDINGS};
         mappings[31] = (EventMapping){settings->hardShowKeysAlt, EVENT_SHOWKEYBINDINGS};
         mappings[32] = (EventMapping){settings->showKeysAlt, EVENT_SHOWKEYBINDINGS};
-        mappings[33] = (EventMapping){settings->hardEndOfPlaylist, EVENT_GOTOENDOFPLAYLIST};
-        mappings[34] = (EventMapping){settings->hardShowTrack, EVENT_SHOWTRACK};
-        mappings[35] = (EventMapping){settings->hardShowTrackAlt, EVENT_SHOWTRACK};
-        mappings[36] = (EventMapping){settings->showTrackAlt, EVENT_SHOWTRACK};
-        mappings[37] = (EventMapping){settings->hardShowLibrary, EVENT_SHOWLIBRARY};
-        mappings[38] = (EventMapping){settings->hardShowLibraryAlt, EVENT_SHOWLIBRARY};
-        mappings[39] = (EventMapping){settings->showLibraryAlt, EVENT_SHOWLIBRARY};
-        mappings[40] = (EventMapping){settings->hardShowSearch, EVENT_SHOWSEARCH};
-        mappings[41] = (EventMapping){settings->hardShowSearchAlt, EVENT_SHOWSEARCH};
-        mappings[42] = (EventMapping){settings->showSearchAlt, EVENT_SHOWSEARCH};
-        mappings[43] = (EventMapping){settings->hardNextPage, EVENT_NEXTPAGE};
-        mappings[44] = (EventMapping){settings->hardPrevPage, EVENT_PREVPAGE};
-        mappings[45] = (EventMapping){settings->hardRemove, EVENT_REMOVE};
-        mappings[46] = (EventMapping){settings->hardRemove2, EVENT_REMOVE};
-        mappings[47] = (EventMapping){settings->tabNext, EVENT_TABNEXT};
+        mappings[33] = (EventMapping){settings->hardShowTrack, EVENT_SHOWTRACK};
+        mappings[34] = (EventMapping){settings->hardShowTrackAlt, EVENT_SHOWTRACK};
+        mappings[35] = (EventMapping){settings->showTrackAlt, EVENT_SHOWTRACK};
+        mappings[36] = (EventMapping){settings->hardShowLibrary, EVENT_SHOWLIBRARY};
+        mappings[37] = (EventMapping){settings->hardShowLibraryAlt, EVENT_SHOWLIBRARY};
+        mappings[38] = (EventMapping){settings->showLibraryAlt, EVENT_SHOWLIBRARY};
+        mappings[39] = (EventMapping){settings->hardShowSearch, EVENT_SHOWSEARCH};
+        mappings[40] = (EventMapping){settings->hardShowSearchAlt, EVENT_SHOWSEARCH};
+        mappings[41] = (EventMapping){settings->showSearchAlt, EVENT_SHOWSEARCH};
+        mappings[42] = (EventMapping){settings->hardNextPage, EVENT_NEXTPAGE};
+        mappings[43] = (EventMapping){settings->hardPrevPage, EVENT_PREVPAGE};
+        mappings[44] = (EventMapping){settings->hardRemove, EVENT_REMOVE};
+        mappings[45] = (EventMapping){settings->hardRemove2, EVENT_REMOVE};
+        mappings[46] = (EventMapping){settings->nextView, EVENT_NEXTVIEW};
+        mappings[47] = (EventMapping){settings->prevView, EVENT_PREVVIEW};
+        mappings[48] = (EventMapping){settings->mouseLeftClick, ui->mouseLeftClickAction};
+        mappings[49] = (EventMapping){settings->mouseMiddleClick, ui->mouseMiddleClickAction};
+        mappings[50] = (EventMapping){settings->mouseRightClick, ui->mouseRightClickAction};
+        mappings[51] = (EventMapping){settings->mouseScrollUp, ui->mouseScrollUpAction};
+        mappings[52] = (EventMapping){settings->mouseScrollDown, ui->mouseScrollDownAction};
+        mappings[53] = (EventMapping){settings->mouseAltScrollUp, ui->mouseAltScrollUpAction};
+        mappings[54] = (EventMapping){settings->mouseAltScrollDown, ui->mouseAltScrollDownAction};
+        mappings[55] = (EventMapping){settings->hardClearPlaylist, EVENT_CLEARPLAYLIST};
+        mappings[56] = (EventMapping){settings->showRadioSearchAlt, EVENT_SHOWRADIOSEARCH};
+        mappings[57] = (EventMapping){settings->hardShowRadioSearch, EVENT_SHOWRADIOSEARCH};
+        mappings[58] = (EventMapping){settings->hardShowRadioSearchAlt, EVENT_SHOWRADIOSEARCH};
+        mappings[59] = (EventMapping){settings->moveSongUp, EVENT_MOVESONGUP};
+        mappings[60] = (EventMapping){settings->moveSongDown, EVENT_MOVESONGDOWN};
+        mappings[61] = (EventMapping){settings->enqueueAndPlay, EVENT_ENQUEUEANDPLAY};
 }
 
 char *getConfigFilePath(char *configdir)
@@ -426,14 +524,14 @@ char *getConfigFilePath(char *configdir)
         if (configdir_length + 1 + settings_file_length + 1 > MAXPATHLEN)
         {
                 fprintf(stderr, "Error: File path exceeds maximum length.\n");
-                exit(EXIT_FAILURE);
+                exit(1);
         }
 
         char *filepath = (char *)malloc(MAXPATHLEN);
         if (filepath == NULL)
         {
                 perror("malloc");
-                exit(EXIT_FAILURE);
+                exit(1);
         }
 
         int written = snprintf(filepath, MAXPATHLEN, "%s/%s", configdir, SETTINGS_FILE);
@@ -441,11 +539,58 @@ char *getConfigFilePath(char *configdir)
         {
                 fprintf(stderr, "Error: snprintf failed or filepath truncated.\n");
                 free(filepath);
-                exit(EXIT_FAILURE);
+                exit(1);
         }
         return filepath;
 }
 
+enum EventType getMouseAction(int num)
+{
+        enum EventType value = EVENT_NONE;
+
+        switch (num)
+        {
+        case 0:
+                value = EVENT_NONE;
+                break;
+        case 1:
+                value = EVENT_GOTOSONG;
+                break;
+        case 2:
+                value = EVENT_PLAY_PAUSE;
+                break;
+        case 3:
+                value = EVENT_SCROLLPREV;
+                break;
+        case 4:
+                value = EVENT_SCROLLNEXT;
+                break;
+        case 5:
+                value = EVENT_SEEKFORWARD;
+                break;
+        case 6:
+                value = EVENT_SEEKBACK;
+                break;
+        case 7:
+                value = EVENT_VOLUME_UP;
+                break;
+        case 8:
+                value = EVENT_VOLUME_DOWN;
+                break;
+        case 9:
+                value = EVENT_NEXTVIEW;
+                break;
+        case 10:
+                value = EVENT_PREVVIEW;
+                break;
+        default:
+                value = EVENT_NONE;
+                break;
+        }
+
+        return value;
+}
+
 void getConfig(AppSettings *settings, UISettings *ui)
 {
         int pair_count;
@@ -458,7 +603,7 @@ void getConfig(AppSettings *settings, UISettings *ui)
                 if (mkdir(configdir, 0700) != 0)
                 {
                         perror("mkdir");
-                        exit(EXIT_FAILURE);
+                        exit(1);
                 }
         }
 
@@ -474,12 +619,15 @@ void getConfig(AppSettings *settings, UISettings *ui)
         ui->coverAnsi = (settings->coverAnsi[0] == '1');
         ui->visualizerEnabled = (settings->visualizerEnabled[0] == '1');
         ui->useConfigColors = (settings->useConfigColors[0] == '1');
+        ui->quitAfterStopping = (settings->quitAfterStopping[0] == '1');
+        ui->hideGlimmeringText = (settings->hideGlimmeringText[0] == '1');
         ui->hideLogo = (settings->hideLogo[0] == '1');
         ui->hideHelp = (settings->hideHelp[0] == '1');
 
         int temp = getNumber(settings->color);
         if (temp >= 0)
                 ui->mainColor = temp;
+
         temp = getNumber(settings->artistColor);
         if (temp >= 0)
                 ui->artistColor = temp;
@@ -492,17 +640,56 @@ void getConfig(AppSettings *settings, UISettings *ui)
         if (temp >= 0)
                 ui->titleColor = temp;
 
-        int temp2 = getNumber(settings->visualizerHeight);
-        if (temp2 > 0)
-                ui->visualizerHeight = temp2;
+        temp = getNumber(settings->mouseLeftClickAction);
+        enum EventType tempEvent = getMouseAction(temp);
+        if (temp >= 0)
+                ui->mouseLeftClickAction = tempEvent;
 
-        int temp3 = getNumber(settings->lastVolume);
-        if (temp3 >= 0)
-                setVolume(temp3);
+        temp = getNumber(settings->mouseMiddleClickAction);
+        tempEvent = getMouseAction(temp);
+        if (temp >= 0)
+                ui->mouseMiddleClickAction = tempEvent;
 
-        int temp4 = getNumber(settings->cacheLibrary);
-        if (temp4 >= 0)
-                ui->cacheLibrary = temp4;
+        temp = getNumber(settings->mouseRightClickAction);
+        tempEvent = getMouseAction(temp);
+        if (temp >= 0)
+                ui->mouseRightClickAction = tempEvent;
+
+        temp = getNumber(settings->mouseScrollUpAction);
+        tempEvent = getMouseAction(temp);
+        if (temp >= 0)
+                ui->mouseScrollUpAction = tempEvent;
+
+        temp = getNumber(settings->mouseScrollDownAction);
+        tempEvent = getMouseAction(temp);
+        if (temp >= 0)
+                ui->mouseScrollDownAction = tempEvent;
+
+        temp = getNumber(settings->mouseAltScrollUpAction);
+        tempEvent = getMouseAction(temp);
+        if (temp >= 0)
+                ui->mouseAltScrollUpAction = tempEvent;
+
+        temp = getNumber(settings->mouseAltScrollDownAction);
+        tempEvent = getMouseAction(temp);
+        if (temp >= 0)
+                ui->mouseAltScrollDownAction = tempEvent;
+
+        temp = getNumber(settings->visualizerHeight);
+        if (temp > 0)
+                ui->visualizerHeight = temp;
+
+        temp = getNumber(settings->titleDelay);
+        if (temp >= 0)
+                ui->titleDelay = temp;
+
+        temp = getNumber(settings->lastVolume);
+        if (temp >= 0)
+                setVolume(temp);
+
+        temp = getNumber(settings->cacheLibrary);
+        if (temp >= 0)
+                ui->cacheLibrary = temp;
 
         getMusicLibraryPath(settings->path);
         free(configdir);
@@ -524,6 +711,7 @@ void setConfig(AppSettings *settings, UISettings *ui)
                 return;
         }
 
+        // Make sure strings are valid before writing settings to the file
         if (settings->allowNotifications[0] == '\0')
                 ui->allowNotifications ? c_strcpy(settings->allowNotifications, "1", sizeof(settings->allowNotifications)) : c_strcpy(settings->allowNotifications, "0", sizeof(settings->allowNotifications));
         if (settings->coverEnabled[0] == '\0')
@@ -534,34 +722,53 @@ void setConfig(AppSettings *settings, UISettings *ui)
                 ui->visualizerEnabled ? c_strcpy(settings->visualizerEnabled, "1", sizeof(settings->visualizerEnabled)) : c_strcpy(settings->visualizerEnabled, "0", sizeof(settings->visualizerEnabled));
         if (settings->useConfigColors[0] == '\0')
                 ui->useConfigColors ? c_strcpy(settings->useConfigColors, "1", sizeof(settings->useConfigColors)) : c_strcpy(settings->useConfigColors, "0", sizeof(settings->useConfigColors));
-        if (settings->visualizerHeight[0] == '\0')
-        {
-                snprintf(settings->visualizerHeight, sizeof(settings->visualizerHeight), "%d", ui->visualizerHeight);
-        }
+        if (settings->quitAfterStopping[0] == '\0')
+                ui->quitAfterStopping ? c_strcpy(settings->quitAfterStopping, "1", sizeof(settings->quitAfterStopping)) : c_strcpy(settings->quitAfterStopping, "0", sizeof(settings->quitAfterStopping));
+        if (settings->hideGlimmeringText[0] == '\0')
+                ui->hideGlimmeringText ? c_strcpy(settings->hideGlimmeringText, "1", sizeof(settings->hideGlimmeringText)) : c_strcpy(settings->hideGlimmeringText, "0", sizeof(settings->hideGlimmeringText));
         if (settings->hideLogo[0] == '\0')
                 ui->hideLogo ? c_strcpy(settings->hideLogo, "1", sizeof(settings->hideLogo)) : c_strcpy(settings->hideLogo, "0", sizeof(settings->hideLogo));
         if (settings->hideHelp[0] == '\0')
                 ui->hideHelp ? c_strcpy(settings->hideHelp, "1", sizeof(settings->hideHelp)) : c_strcpy(settings->hideHelp, "0", sizeof(settings->hideHelp));
 
-        snprintf(settings->cacheLibrary, sizeof(settings->cacheLibrary), "%d", ui->cacheLibrary);
+        if (settings->visualizerHeight[0] == '\0')
+                snprintf(settings->visualizerHeight, sizeof(settings->visualizerHeight), "%d", ui->visualizerHeight);
+        if (settings->titleDelay[0] == '\0')
+                snprintf(settings->titleDelay, sizeof(settings->titleDelay), "%d", ui->titleDelay);
+        if (settings->cacheLibrary[0] == '\0')
+                snprintf(settings->cacheLibrary, sizeof(settings->cacheLibrary), "%d", ui->cacheLibrary);
 
-        int currentVolume = getCurrentVolume();
-        currentVolume = (currentVolume <= 0) ? 10 : currentVolume;
+        if (settings->lastVolume[0] == '\0')
+        {
+                int currentVolume = getCurrentVolume();
+                currentVolume = (currentVolume <= 0) ? 10 : currentVolume;
+                snprintf(settings->lastVolume, sizeof(settings->lastVolume), "%d", currentVolume);
+        }
 
-        snprintf(settings->lastVolume, sizeof(settings->lastVolume), "%d", currentVolume);
 
-        // Null-terminate the character arrays
-        settings->path[MAXPATHLEN - 1] = '\0';
-        settings->coverEnabled[1] = '\0';
-        settings->coverAnsi[1] = '\0';
-        settings->visualizerEnabled[1] = '\0';
-        settings->visualizerHeight[5] = '\0';
-        settings->lastVolume[5] = '\0';
-        settings->useConfigColors[1] = '\0';
-        settings->allowNotifications[1] = '\0';
-        settings->hideLogo[1] = '\0';
-        settings->hideHelp[1] = '\0';
-        settings->cacheLibrary[5] = '\0';
+        if (settings->color[0] == '\0')
+                snprintf(settings->color, sizeof(settings->color), "%d", ui->mainColor);
+        if (settings->artistColor[0] == '\0')
+                snprintf(settings->artistColor, sizeof(settings->artistColor), "%d", ui->artistColor);
+        if (settings->titleColor[0] == '\0')
+                snprintf(settings->titleColor, sizeof(settings->titleColor), "%d", ui->titleColor);
+        if (settings->enqueuedColor[0] == '\0')
+                snprintf(settings->enqueuedColor, sizeof(settings->enqueuedColor), "%d", ui->enqueuedColor);
+
+        if (settings->mouseLeftClickAction[0] == '\0')
+                snprintf(settings->mouseLeftClickAction, sizeof(settings->mouseLeftClickAction), "%d", ui->mouseLeftClickAction);
+        if (settings->mouseMiddleClickAction[0] == '\0')
+                snprintf(settings->mouseMiddleClickAction, sizeof(settings->mouseMiddleClickAction), "%d", ui->mouseMiddleClickAction);
+        if (settings->mouseRightClickAction[0] == '\0')
+                snprintf(settings->mouseRightClickAction, sizeof(settings->mouseRightClickAction), "%d", ui->mouseRightClickAction);
+        if (settings->mouseScrollUpAction[0] == '\0')
+                snprintf(settings->mouseScrollUpAction, sizeof(settings->mouseScrollUpAction), "%d", ui->mouseScrollUpAction);
+        if (settings->mouseScrollDownAction[0] == '\0')
+                snprintf(settings->mouseScrollDownAction, sizeof(settings->mouseScrollDownAction), "%d", ui->mouseScrollDownAction);
+        if (settings->mouseAltScrollUpAction[0] == '\0')
+                snprintf(settings->mouseAltScrollUpAction, sizeof(settings->mouseAltScrollUpAction), "%d", ui->mouseAltScrollUpAction);
+        if (settings->mouseAltScrollDownAction[0] == '\0')
+                snprintf(settings->mouseAltScrollDownAction, sizeof(settings->mouseAltScrollDownAction), "%d", ui->mouseAltScrollDownAction);
 
         // Write the settings to the file
         fprintf(file, "# Make sure that kew is closed before editing this file in order for changes to take effect.\n\n");
@@ -580,6 +787,15 @@ void setConfig(AppSettings *settings, UISettings *ui)
         fprintf(file, "\n# Cache: Set to 1 to use cache of the music library directory tree for faster startup times.\n");
         fprintf(file, "cacheLibrary=%s\n", settings->cacheLibrary);
 
+        fprintf(file, "\n# Delay when drawing title in track view, set to 0 to have no delay.\n");
+        fprintf(file, "titleDelay=%s\n", settings->titleDelay);
+
+        fprintf(file, "\n# Same as '--quitonstop' flag, exits after playing the whole playlist.\n");
+        fprintf(file, "quitOnStop=%s\n", settings->quitAfterStopping);
+
+        fprintf(file, "\n# Glimmering text on the bottom row.\n");
+        fprintf(file, "hideGlimmeringText=%s\n", settings->hideGlimmeringText);
+
         fprintf(file, "\n# Color values are 0=Black, 1=Red, 2=Green, 3=Yellow, 4=Blue, 5=Magenta, 6=Cyan, 7=White\n");
         fprintf(file, "# These mostly affect the library view.\n\n");
         fprintf(file, "# Logo color:\n");
@@ -591,6 +807,16 @@ void setConfig(AppSettings *settings, UISettings *ui)
         fprintf(file, "# Color of enqueued songs in library view:\n");
         fprintf(file, "enqueuedColor=%s\n", settings->enqueuedColor);
 
+        fprintf(file, "\n# Mouse actions are 0=none, 1=select song, 2=toggle pause, 3=scroll up, 4=scroll down, 5=seek forward, 6=seek backward, 7=volume up, 8=volume down, 9=switch to next view, 10=switch to previous view\n");
+        fprintf(file, "mouseLeftClickAction=%s\n", settings->mouseLeftClickAction);
+        fprintf(file, "mouseMiddleClickAction=%s\n", settings->mouseMiddleClickAction);
+        fprintf(file, "mouseRightClickAction=%s\n", settings->mouseRightClickAction);
+        fprintf(file, "mouseScrollUpAction=%s\n", settings->mouseScrollUpAction);
+        fprintf(file, "mouseScrollDownAction=%s\n", settings->mouseScrollDownAction);
+        fprintf(file, "# Mouse action when using mouse scroll + alt\n");
+        fprintf(file, "mouseAltScrollUpAction=%s\n", settings->mouseAltScrollUpAction);
+        fprintf(file, "mouseAltScrollDownAction=%s\n", settings->mouseAltScrollDownAction);
+
         fprintf(file, "\n# Key Bindings:\n\n");
         fprintf(file, "volumeUp=%s\n", settings->volumeUp);
         fprintf(file, "volumeUpAlt=%s\n", settings->volumeUpAlt);
@@ -611,11 +837,16 @@ void setConfig(AppSettings *settings, UISettings *ui)
         fprintf(file, "savePlaylist=%s\n", settings->savePlaylist);
         fprintf(file, "addToMainPlaylist=%s\n", settings->addToMainPlaylist);
         fprintf(file, "updateLibrary=%s\n", settings->updateLibrary);
-        fprintf(file, "\n# The different main views, normally F2-F6:\n");
+        fprintf(file, "moveSongUp=%s\n", settings->moveSongUp);
+        fprintf(file, "moveSongDown=%s\n", settings->moveSongDown);
+        fprintf(file, "enqueueAndPlay=%s\n", settings->enqueueAndPlay);
+
+        fprintf(file, "\n# Alt keys for the different main views, normally F2-F7:\n");
         fprintf(file, "showPlaylistAlt=%s\n", settings->showPlaylistAlt);
         fprintf(file, "showLibraryAlt=%s\n", settings->showLibraryAlt);
         fprintf(file, "showTrackAlt=%s\n", settings->showTrackAlt);
         fprintf(file, "showSearchAlt=%s\n", settings->showSearchAlt);
+        fprintf(file, "showRadioSearchAlt=%s\n\n", settings->showRadioSearchAlt);
         fprintf(file, "showKeysAlt=%s\n\n", settings->showKeysAlt);
 
         fprintf(file, "quit=%s\n\n", settings->quit);
