@@ -8,6 +8,8 @@
 #include <stdlib.h>
 #include <sys/param.h>
 #include <unistd.h>
+#include "appstate.h"
+#include "events.h"
 #include "file.h"
 #include "soundcommon.h"
 #include "player.h"
@@ -17,12 +19,18 @@
 #define MAXPATHLEN 4096
 #endif
 
-extern time_t lastTimeAppRan;
+#ifndef NUM_KEY_MAPPINGS
+#define NUM_KEY_MAPPINGS 62
+#endif
 
 extern AppSettings settings;
 
-void getConfig(AppSettings *settings);
+enum EventType getMouseAction(int num);
 
-void setConfig(AppSettings *settings);
+void getConfig(AppSettings *settings, UISettings *ui);
+
+void setConfig(AppSettings *settings, UISettings *ui);
+
+void mapSettingsToKeys(AppSettings *settings, UISettings *ui, EventMapping *mappings);
 
 #endif
