@@ -1,11 +1,11 @@
-#include <FreeImage.h>
 #include <glib.h>
 #include <gio/gio.h>
 #include <sys/wait.h>
 #include <unistd.h>
-#include <libavformat/avformat.h>
+#include "appstate.h"
+#include "tagLibWrapper.h"
 #include "cache.h"
-#include "chafafunc.h"
+#include "imgfunc.h"
 #include "file.h"
 #include "sound.h"
 #include "soundcommon.h"
@@ -29,13 +29,15 @@ typedef struct
 #ifndef TAGSETTINGS_STRUCT
 #define TAGSETTINGS_STRUCT
 
+#define METADATA_MAX_SIZE 256
+
 typedef struct
 {
-        char title[256];
-        char artist[256];
-        char album_artist[256];
-        char album[256];
-        char date[256];
+        char title[METADATA_MAX_SIZE];
+        char artist[METADATA_MAX_SIZE];
+        char album_artist[METADATA_MAX_SIZE];
+        char album[METADATA_MAX_SIZE];
+        char date[METADATA_MAX_SIZE];
 } TagSettings;
 
 #endif
@@ -52,14 +54,15 @@ typedef struct
         unsigned char green;
         unsigned char blue;
         TagSettings *metadata;
-        FIBITMAP *cover;
+        unsigned char *cover;
+        int coverWidth;
+        int coverHeight;
         double duration;
         bool hasErrors;
 } SongData;
 
 #endif
 
-extern Cache *tempCache;
+SongData *loadSongData(char *filePath, AppState *state);
 
-SongData *loadSongData(char *filePath);
-void unloadSongData(SongData **songdata);
+void unloadSongData(SongData **songdata, AppState *state);
