@@ -3,7 +3,6 @@
 #define MINIAUDIO_IMPLEMENTATION
 
 #include <miniaudio.h>
-#include "mpris.h"
 #include "sound.h"
 
 /*
@@ -20,25 +19,18 @@ bool isContextInitialized = false;
 
 UserData userData;
 
-int check_aac_codec_support()
-{
-        const AVCodec *codec = avcodec_find_decoder(AV_CODEC_ID_AAC);
-
-        if (codec == NULL)
-        {
-                fprintf(stderr, "AAC codec not supported in this build of FFmpeg.\n");
-
-                return -1;
-        }
-
-        return 0;
-}
-
 ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
 {
         char *filePath = NULL;
 
-        filePath = (pAudioData->currentFileIndex == 0) ? pUserData->songdataA->filePath : pUserData->songdataB->filePath;
+        SongData *songData = (pAudioData->currentFileIndex == 0) ? pUserData->songdataA : pUserData->songdataB;
+
+        if (songData == NULL)
+        {
+                return MA_ERROR;
+        }
+
+        filePath = songData->filePath;
 
         pAudioData->pUserData = pUserData;
         pAudioData->currentPCMFrame = 0;
@@ -55,7 +47,7 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                 pAudioData->sampleRate = first->outputSampleRate;
                 ma_data_source_get_length_in_pcm_frames(first, &pAudioData->totalFrames);
         }
-        else if (endsWith(filePath, "opus"))
+        else if (pathEndsWith(filePath, "opus"))
         {
                 int result = prepareNextOpusDecoder(filePath);
                 if (result < 0)
@@ -68,7 +60,7 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
         }
-        else if (endsWith(filePath, "ogg"))
+        else if (pathEndsWith(filePath, "ogg"))
         {
                 int result = prepareNextVorbisDecoder(filePath);
                 if (result < 0)
@@ -81,9 +73,11 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
         }
-        else if (endsWith(filePath, "m4a") || endsWith(filePath, "aac"))
+        else if (pathEndsWith(filePath, "m4a") || pathEndsWith(filePath, "aac"))
         {
-                int result = prepareNextM4aDecoder(filePath);
+#ifdef USE_FAAD
+
+                int result = prepareNextM4aDecoder(songData);
                 if (result < 0)
                         return -1;
                 m4a_decoder *first = getFirstM4aDecoder();
@@ -93,6 +87,9 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                 ma_data_source_base *base = (ma_data_source_base *)first;
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
+#else
+                return MA_ERROR;
+#endif
         }
         else
         {
@@ -129,9 +126,10 @@ int createDevice(UserData *userData, ma_device *device, ma_context *context, ma_
         result = ma_device_start(device);
         if (result != MA_SUCCESS)
                 return -1;
-        emitStringPropertyChanged("PlaybackStatus", "Playing");
 
-        return 0;        
+        appState.uiState.doNotifyMPRISPlaying = true;
+
+        return 0;
 }
 
 int builtin_createAudioDevice(UserData *userData, ma_device *device, ma_context *context, ma_data_source_vtable *vtable)
@@ -161,7 +159,7 @@ int vorbis_createAudioDevice(UserData *userData, ma_device *device, ma_context *
         result = ma_device_init(context, &deviceConfig, device);
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to initialize miniaudio device.\n");
+                setErrorMessage("Failed to initialize miniaudio device.");
                 return -1;
         }
 
@@ -170,14 +168,16 @@ int vorbis_createAudioDevice(UserData *userData, ma_device *device, ma_context *
         result = ma_device_start(device);
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to start miniaudio device.\n");
+                setErrorMessage("Failed to start miniaudio device.");
                 return -1;
         }
-        emitStringPropertyChanged("PlaybackStatus", "Playing");
 
-        return 0;        
+        appState.uiState.doNotifyMPRISPlaying = true;
+
+        return 0;
 }
 
+#ifdef USE_FAAD
 int m4a_createAudioDevice(UserData *userData, ma_device *device, ma_context *context)
 {
         ma_result result;
@@ -185,9 +185,9 @@ int m4a_createAudioDevice(UserData *userData, ma_device *device, ma_context *con
         result = initFirstDatasource(&audioData, userData);
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to initialize m4a file.\n");
+                setErrorMessage("M4a type not supported.");
                 return -1;
-        }        
+        }
         m4a_decoder *decoder = getFirstM4aDecoder();
         ma_device_config deviceConfig = ma_device_config_init(ma_device_type_playback);
 
@@ -200,7 +200,7 @@ int m4a_createAudioDevice(UserData *userData, ma_device *device, ma_context *con
         result = ma_device_init(context, &deviceConfig, device);
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to initialize miniaudio device.\n");
+                setErrorMessage("Failed to initialize miniaudio device.");
                 return -1;
         }
 
@@ -210,13 +210,15 @@ int m4a_createAudioDevice(UserData *userData, ma_device *device, ma_context *con
 
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to start miniaudio device.\n");
+                setErrorMessage("Failed to start miniaudio device.");
                 return -1;
         }
-        emitStringPropertyChanged("PlaybackStatus", "Playing");
 
-        return 0;        
+        appState.uiState.doNotifyMPRISPlaying = true;
+
+        return 0;
 }
+#endif
 
 int opus_createAudioDevice(UserData *userData, ma_device *device, ma_context *context)
 {
@@ -227,7 +229,7 @@ int opus_createAudioDevice(UserData *userData, ma_device *device, ma_context *co
         {
                 printf("\n\nFailed to initialize opus file.\n");
                 return -1;
-        }        
+        }
         ma_libopus *opus = getFirstOpusDecoder();
 
         ma_device_config deviceConfig = ma_device_config_init(ma_device_type_playback);
@@ -241,7 +243,7 @@ int opus_createAudioDevice(UserData *userData, ma_device *device, ma_context *co
         result = ma_device_init(context, &deviceConfig, device);
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to initialize miniaudio device.\n");
+                setErrorMessage("Failed to initialize miniaudio device.");
                 return -1;
         }
 
@@ -250,10 +252,11 @@ int opus_createAudioDevice(UserData *userData, ma_device *device, ma_context *co
         result = ma_device_start(device);
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to start miniaudio device.\n");
+                setErrorMessage("Failed to start miniaudio device.");
                 return -1;
         }
-        emitStringPropertyChanged("PlaybackStatus", "Playing");
+
+        appState.uiState.doNotifyMPRISPlaying = true;
 
         return 0;
 }
@@ -271,8 +274,13 @@ bool validFilePath(char *filePath)
 
 bool tryAgain = false;
 
-int switchAudioImplementation()
+int switchAudioImplementation(void)
 {
+        if (isRadioPlaying())
+        {
+                stopRadio();
+        }
+
         if (audioData.endOfListReached)
         {
                 setEOFNotReached();
@@ -293,7 +301,7 @@ int switchAudioImplementation()
         }
         else
         {
-                
+
                 if (!validFilePath(userData.currentSongData->filePath))
                 {
                         if (!tryAgain)
@@ -338,10 +346,7 @@ int switchAudioImplementation()
 
                         cleanupPlaybackDevice();
 
-                        resetDecoders();
-                        resetVorbisDecoders();
-                        resetM4aDecoders();
-                        resetOpusDecoders();
+                        resetAllDecoders();
                         resetAudioBuffer();
 
                         int result = builtin_createAudioDevice(&userData, getDevice(), &context, &builtin_file_data_source_vtable);
@@ -352,7 +357,7 @@ int switchAudioImplementation()
                                 setImplSwitchNotReached();
                                 setEOFReached();
                                 free(filePath);
-                                pthread_mutex_unlock(&dataSourceMutex);                                
+                                pthread_mutex_unlock(&dataSourceMutex);
                                 return -1;
                         }
 
@@ -361,7 +366,7 @@ int switchAudioImplementation()
                         setImplSwitchNotReached();
                 }
         }
-        else if (endsWith(filePath, "opus"))
+        else if (pathEndsWith(filePath, "opus"))
         {
                 ma_uint32 sampleRate;
                 ma_uint32 channels;
@@ -393,10 +398,7 @@ int switchAudioImplementation()
 
                         cleanupPlaybackDevice();
 
-                        resetDecoders();
-                        resetVorbisDecoders();
-                        resetM4aDecoders();
-                        resetOpusDecoders();
+                        resetAllDecoders();
                         resetAudioBuffer();
 
                         int result = opus_createAudioDevice(&userData, getDevice(), &context);
@@ -407,7 +409,7 @@ int switchAudioImplementation()
                                 setImplSwitchNotReached();
                                 setEOFReached();
                                 free(filePath);
-                                pthread_mutex_unlock(&dataSourceMutex);                                
+                                pthread_mutex_unlock(&dataSourceMutex);
                                 return -1;
                         }
 
@@ -416,7 +418,7 @@ int switchAudioImplementation()
                         setImplSwitchNotReached();
                 }
         }
-        else if (endsWith(filePath, "ogg"))
+        else if (pathEndsWith(filePath, "ogg"))
         {
                 ma_uint32 sampleRate;
                 ma_uint32 channels;
@@ -448,10 +450,7 @@ int switchAudioImplementation()
 
                         cleanupPlaybackDevice();
 
-                        resetDecoders();
-                        resetVorbisDecoders();
-                        resetM4aDecoders();
-                        resetOpusDecoders();
+                        resetAllDecoders();
                         resetAudioBuffer();
 
                         int result = vorbis_createAudioDevice(&userData, getDevice(), &context);
@@ -462,7 +461,7 @@ int switchAudioImplementation()
                                 setImplSwitchNotReached();
                                 setEOFReached();
                                 free(filePath);
-                                pthread_mutex_unlock(&dataSourceMutex);                                
+                                pthread_mutex_unlock(&dataSourceMutex);
                                 return -1;
                         }
 
@@ -471,15 +470,9 @@ int switchAudioImplementation()
                         setImplSwitchNotReached();
                 }
         }
-        else if (endsWith(filePath, "m4a") || endsWith(filePath, "aac"))
+        else if (pathEndsWith(filePath, "m4a") || pathEndsWith(filePath, "aac"))
         {
-                if (check_aac_codec_support() < 0)
-                {
-                        free(filePath);
-                        printf("\n\nUnable to find AAC codec. If you have the free version of FFmpeg, there might be no AAC/M4A file support.\n");
-                        exit(0);
-                }
-
+#ifdef USE_FAAD
                 ma_uint32 sampleRate;
                 ma_uint32 channels;
                 ma_format format;
@@ -490,15 +483,18 @@ int switchAudioImplementation()
                 ma_format nFormat;
                 ma_channel nChannelMap[MA_MAX_CHANNELS];
                 m4a_decoder *decoder = getCurrentM4aDecoder();
+                k_m4adec_filetype fileType = k_unknown;
 
-                getM4aFileInfo(filePath, &format, &channels, &sampleRate, channelMap);
+                getM4aFileInfo(filePath, &format, &channels, &sampleRate, channelMap, &fileType);
 
                 if (decoder != NULL)
                         m4a_decoder_ds_get_data_format(decoder, &nFormat, &nChannels, &nSampleRate, nChannelMap, MA_MAX_CHANNELS);
 
                 bool sameFormat = (decoder != NULL && (format == decoder->format &&
                                                        channels == nChannels &&
-                                                       sampleRate == nSampleRate));
+                                                       sampleRate == nSampleRate &&
+                                                        decoder->fileType == fileType &&
+                                                        decoder->fileType != k_rawAAC));
 
                 if (isRepeatEnabled() || !(sameFormat && currentImplementation == M4A))
                 {
@@ -510,10 +506,7 @@ int switchAudioImplementation()
 
                         cleanupPlaybackDevice();
 
-                        resetDecoders();
-                        resetVorbisDecoders();
-                        resetM4aDecoders();
-                        resetOpusDecoders();
+                        resetAllDecoders();
                         resetAudioBuffer();
 
                         int result = m4a_createAudioDevice(&userData, getDevice(), &context);
@@ -524,7 +517,7 @@ int switchAudioImplementation()
                                 setImplSwitchNotReached();
                                 setEOFReached();
                                 free(filePath);
-                                pthread_mutex_unlock(&dataSourceMutex);                                
+                                pthread_mutex_unlock(&dataSourceMutex);
                                 return -1;
                         }
 
@@ -532,6 +525,14 @@ int switchAudioImplementation()
 
                         setImplSwitchNotReached();
                 }
+#else
+                setCurrentImplementationType(NONE);
+                setImplSwitchNotReached();
+                setEOFReached();
+                free(filePath);
+                pthread_mutex_unlock(&dataSourceMutex);
+                return -1;
+#endif
         }
         else
         {
@@ -545,13 +546,13 @@ int switchAudioImplementation()
         return 0;
 }
 
-void cleanupAudioContext()
+void cleanupAudioContext(void)
 {
         ma_context_uninit(&context);
         isContextInitialized = false;
 }
 
-int createAudioDevice(UserData *userData)
+int createAudioDevice()
 {
         if (isContextInitialized)
         {
@@ -563,24 +564,7 @@ int createAudioDevice(UserData *userData)
 
         if (switchAudioImplementation() >= 0)
         {
-                SongData *currentSongData = userData->currentSongData;
-
-                if (currentSongData != NULL && currentSongData->hasErrors == 0 && currentSongData->metadata && strlen(currentSongData->metadata->title) > 0)
-                {
-#ifdef USE_LIBNOTIFY
-                        displaySongNotification(currentSongData->metadata->artist, currentSongData->metadata->title, currentSongData->coverArtPath);
-#endif
-
-                        gint64 length = getLengthInMicroSec(currentSongData->duration);
-                        // update mpris
-                        emitMetadataChanged(
-                            currentSongData->metadata->title,
-                            currentSongData->metadata->artist,
-                            currentSongData->metadata->album,
-                            currentSongData->coverArtPath,
-                            currentSongData->trackId != NULL ? currentSongData->trackId : "", currentSong,
-                            length);
-                }
+                appState.uiState.doNotifyMPRISSwitched = true;
         }
         else
         {
