@@ -16,6 +16,8 @@
 #include "songloader.h"
 #include "soundbuiltin.h"
 #include "soundcommon.h"
+#include "soundradio.h"
+#include "common.h"
 
 #ifndef USERDATA_STRUCT
 #define USERDATA_STRUCT
@@ -44,30 +46,18 @@ typedef struct
         int currentFileIndex;
         ma_uint64 totalFrames;
         bool endOfListReached;
-        bool restart;     
+        bool restart;
 } AudioData;
 #endif
 
-extern AudioData audioData;
-
 extern UserData userData;
 
 extern bool isContextInitialized;
 
-int prepareNextDecoder(char *filepath);
+int createAudioDevice();
 
-int prepareNextOpusDecoder(char *filepath);
+int switchAudioImplementation(void);
 
-int prepareNextVorbisDecoder(char *filepath);
-
-int prepareNextM4aDecoder(char *filepath);
-
-void setDecoders(bool usingA, char *filePath);
-
-int createAudioDevice(UserData *userData);
-
-int switchAudioImplementation();
-
-void cleanupAudioContext();
+void cleanupAudioContext(void);
 
 #endif
