@@ -9,8 +9,6 @@ term.c
 
 */
 
-volatile sig_atomic_t resizeFlag = 0;
-
 void setTextColor(int color)
 {
         /*
@@ -110,7 +108,7 @@ void resetConsole()
         printf("\033\143");     // Reset to Initial State (RIS)
         printf("\033[3J");      // Clear scrollback buffer
         printf("\033[H\033[J"); // Move cursor to top-left and clear screen
-        
+
         fflush(stdout);
 }
 
@@ -122,7 +120,7 @@ void clearRestOfScreen()
 void clearScreen()
 {
         printf("\033[3J");      // Clear scrollback buffer
-        printf("\033[H\033[J"); // Move cursor to top-left and clear screen
+        printf("\033[2J\033[3J\033[H"); // Move cursor to top-left and clear screen and scrollback buffer
 }
 
 void enableScrolling()
@@ -130,29 +128,6 @@ void enableScrolling()
         printf("\033[?7h");
 }
 
-void handleResize(int sig)
-{
-        (void)sig;
-        resizeFlag = 1;
-}
-
-void resetResizeFlag(int sig)
-{
-        (void)sig;
-        resizeFlag = 0;
-}
-
-void initResize()
-{
-        signal(SIGWINCH, handleResize);
-
-        struct sigaction sa;
-        sa.sa_handler = resetResizeFlag;
-        sigemptyset(&sa.sa_mask);
-        sa.sa_flags = 0;
-        sigaction(SIGALRM, &sa, NULL);
-}
-
 void disableInputBuffering(void)
 {
         struct termios term;
@@ -180,40 +155,6 @@ void cursorJumpDown(int numRows)
         printf("\033[%dB", numRows);
 }
 
-int readInputSequence_old(char *seq, size_t seqSize)
-{
-        char c;
-        ssize_t bytesRead;
-        seq[0] = '\0';
-        bytesRead = read(STDIN_FILENO, &c, 1);
-        if (bytesRead <= 0)
-        {
-                return 0;
-        }
-
-        // If it's not an escape character, return it as a single-character sequence
-        if (c != '\x1b')
-        {
-                seq[0] = c;
-                seq[1] = '\0';
-                return 1;
-        }
-
-        // Read additional characters
-        if (seqSize < 3)
-                return 0;
-
-        bytesRead = read(STDIN_FILENO, seq, 2);
-        if ((bytesRead <= 0) & (seq[0] == '\0'))
-        {
-                seq[0] = '\0';
-                return 0;
-        }
-
-        seq[bytesRead] = '\0';
-        return bytesRead;
-}
-
 int readInputSequence(char *seq, size_t seqSize)
 {
         char c;
@@ -279,59 +220,26 @@ int getIndentation(int terminalWidth)
         return (indent > 0) ? indent : 0;
 }
 
-int isFunctionKey(const char *seq)
+void enterAlternateScreenBuffer()
 {
-        if (seq[0] == '\033')
-        { // ESC character
-                return 1;
-        }
-        return 0;
+        // Enter alternate screen buffer
+        printf("\033[?1049h");
 }
 
-void convertControlNotationToAscii(char *input, char *output, size_t size)
+void exitAlternateScreenBuffer()
 {
-        if (input[0] == '^' && input[1] >= 'A' && input[1] <= 'Z')
-        {
-                output[0] = input[1] - 'A' + 1; // Convert ^A to Ctrl+A (ASCII 1)
-                output[1] = '\0';
-        }
-        else if (input[0] == '^' && input[1] >= '2' && input[1] <= '8')
-        {
-                output[0] = input[1] - '1' + 1; // Convert ^2 to Ctrl+2 (ASCII 18), etc.
-                output[1] = '\0';
-        }
-        else
-        {
-                strncpy(output, input, size);
-        }
+        // Exit alternate screen buffer
+        printf("\033[?1049l");
 }
 
-void convertAsciiToControlNotation(char input, char *output, size_t size)
+void enableMouseButtons()
 {
-        if (input >= 1 && input <= 26)
-        { // Control characters from Ctrl+A to Ctrl+Z
-                if (size >= 3)
-                { // Ensure there's enough space for "^A" and null terminator
-                        output[0] = '^';
-                        output[1] = input + 'A' - 1;
-                        output[2] = '\0';
-                }
-        }
-        else if (input >= 1 && input <= 8)
-        { // Control characters for digits and punctuation (Ctrl+2 to Ctrl+8)
-                if (size >= 3)
-                { // Ensure there's enough space for "^2" and null terminator
-                        output[0] = '^';
-                        output[1] = input + '1' - 1;
-                        output[2] = '\0';
-                }
-        }
-        else
-        {
-                if (size >= 2)
-                { // Ensure there's enough space for the character and null terminator
-                        output[0] = input;
-                        output[1] = '\0';
-                }
-        }
+        // Enable program to accept mouse input as codes
+        printf("\033[?1002h");
+}
+
+void disableMouseButtons()
+{
+        // Disable program to accept mouse input as codes
+        printf("\033[?1002l");
 }
