@@ -66,4 +66,12 @@ void clearScreen(void);
 
 int readInputSequence(char *seq, size_t seqSize);
 
+void enterAlternateScreenBuffer(void);
+
+void exitAlternateScreenBuffer(void);
+
+void enableMouseButtons(void);
+
+void disableMouseButtons(void);
+
 #endif
