@@ -9,14 +9,17 @@ visuals.c
 */
 
 #ifndef MAX_BUFFER_SIZE
-#define MAX_BUFFER_SIZE 4800
+#define MAX_BUFFER_SIZE 8192
 #endif
 
-int bufferSize = 4800;
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
+
+int bufferSize = 8192;
 int prevBufferSize = 0;
 float alpha = 0.2f;
 float lastMax = -1.0f;
-bool unicodeSupport = false;
 fftwf_complex *fftInput = NULL;
 fftwf_complex *fftOutput = NULL;
 
@@ -27,23 +30,6 @@ float lastMagnitudes[MAX_BUFFER_SIZE] = {0.0f};
 float smoothedMagnitudes[MAX_BUFFER_SIZE];
 float maxPossibleMagnitude = 0;
 
-int terminalSupportsUnicode()
-{
-        char *locale = setlocale(LC_ALL, "");
-        if (locale != NULL)
-                return 1;
-
-        return 0;
-}
-
-void initVisuals()
-{
-        unicodeSupport = false;
-
-        if (terminalSupportsUnicode() > 0)
-                unicodeSupport = true;
-}
-
 #define MOVING_AVERAGE_WINDOW_SIZE 2
 
 void updateMagnitudes(int height, int width, float maxMagnitude, float *magnitudes)
@@ -52,7 +38,7 @@ void updateMagnitudes(int height, int width, float maxMagnitude, float *magnitud
         {
                 maxMagnitude = 1.0f;
         }
-        
+
         // Apply moving average smoothing to magnitudes
         for (int i = 0; i < width; i++)
         {
@@ -227,10 +213,10 @@ void calc(int height, int numBars, ma_int32 *audioBuffer, int bitDepth, fftwf_co
         enhancePeaks(numBars, magnitudes, height);
 }
 
-wchar_t *upwardMotionChars[] = {
-    L" ", L"▁", L"▂", L"▃", L"▄", L"▅", L"▆", L"▇", L"█"};
+char *upwardMotionChars[] = {
+    " ", "▁", "▂", "▃", "▄", "▅", "▆", "▇", "█"};
 
-wchar_t *getUpwardMotionChar(int level)
+char *getUpwardMotionChar(int level)
 {
         if (level < 0 || level > 8)
         {
@@ -288,10 +274,9 @@ PixelData increaseLuminosity(PixelData pixel, int amount)
         return pixel2;
 }
 
-void printSpectrum(int height, int width, float *magnitudes, PixelData color, int indentation, bool useProfileColors)
+void printSpectrum(int height, int width, float *magnitudes, PixelData color, int indentation, bool useConfigColors)
 {
         printf("\n");
-        clearRestOfScreen();
 
         PixelData tmp;
 
@@ -301,7 +286,7 @@ void printSpectrum(int height, int width, float *magnitudes, PixelData color, in
                 printBlankSpaces(indentation);
                 if (color.r != 0 || color.g != 0 || color.b != 0)
                 {
-                        if (!useProfileColors)
+                        if (!useConfigColors)
                         {
                                 tmp = increaseLuminosity(color, round(j * height * 4));
                                 printf("\033[38;2;%d;%d;%dm", tmp.r, tmp.g, tmp.b);
@@ -311,40 +296,31 @@ void printSpectrum(int height, int width, float *magnitudes, PixelData color, in
                 {
                         setDefaultTextColor();
                 }
+
                 if (isPaused() || isStopped())
                 {
                         for (int i = 0; i < width; i++)
                         {
                                 printf("  ");
                         }
+                        printf("\n ");
+                        continue;
                 }
-                else
+
+                for (int i = 0; i < width; i++)
                 {
-                        for (int i = 0; i < width; i++)
+                        if (magnitudes[i] >= j)
                         {
-                                if (j >= 0)
-                                {
-                                        if (magnitudes[i] >= j)
-                                        {
-                                                if (unicodeSupport)
-                                                {
-                                                        printf(" %S", getUpwardMotionChar(10));
-                                                }
-                                                else
-                                                {
-                                                        printf(" █");
-                                                }
-                                        }
-                                        else if (magnitudes[i] + 1 >= j && unicodeSupport)
-                                        {
-                                                int firstDecimalDigit = (int)(fmod(magnitudes[i] * 10, 10));
-                                                printf(" %S", getUpwardMotionChar(firstDecimalDigit));
-                                        }
-                                        else
-                                        {
-                                                printf("  ");
-                                        }
-                                }
+                                printf(" %s", getUpwardMotionChar(10));
+                        }
+                        else if (magnitudes[i] + 1 >= j)
+                        {
+                                int firstDecimalDigit = (int)(fmod(magnitudes[i] * 10, 10));
+                                printf(" %s", getUpwardMotionChar(firstDecimalDigit));
+                        }
+                        else
+                        {
+                                printf("  ");
                         }
                 }
                 printf("\n ");
@@ -353,7 +329,7 @@ void printSpectrum(int height, int width, float *magnitudes, PixelData color, in
         fflush(stdout);
 }
 
-void freeVisuals()
+void freeVisuals(void)
 {
         if (fftInput != NULL)
         {
@@ -367,7 +343,7 @@ void freeVisuals()
         }
 }
 
-void drawSpectrumVisualizer(int height, int width, PixelData c, int indentation, bool useProfileColors)
+void drawSpectrumVisualizer(int height, int width, PixelData c, int indentation, bool useConfigColors)
 {
         bufferSize = getBufferSize();
         PixelData color;
@@ -418,7 +394,7 @@ void drawSpectrumVisualizer(int height, int width, PixelData c, int indentation,
 
         calcSpectrum(height, numBars, fftInput, fftOutput, magnitudes, plan);
 
-        printSpectrum(height, numBars, magnitudes, color, indentation, useProfileColors);
+        printSpectrum(height, numBars, magnitudes, color, indentation, useConfigColors);
 
         fftwf_destroy_plan(plan);
 }
